/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 * @brief Definition of CEdt
 */

#ifndef _EDT_KERNEL_STRUCT_H_
#define _EDT_KERNEL_STRUCT_H_

#include "edt_mem_mapping.h"
#include "edtdrv.h"
#include "edt_trace.h"
#include "edt_ioctl.h"  // EDT_DEPSIZE and other "size" macros
#include "edt_shared_types.h"

#ifdef __cplusplus
extern "C" {
#endif

/* initial 1 MB tracebuf 128k entries */

/* #define EDT_INITIAL_TRACESIZE	0x40000  */
#define EDT_INITIAL_TRACESIZE 0x400

#define MAX_MINORS 34

#define DYNAMIC_RBUFS

#ifndef PDV_SSIZE

#ifdef PDV
#define PDV_SSIZE 2048
#else
#define PDV_SSIZE 2
#endif

#endif

// Has to be 4 for PCI boards with burst disabled; min 128 (rev06)
// or 64 (rev07) for pe8lx.  -mcm  Feb 2012
#define EDT_MIN_TRANSFER 4

/* this define controls the length of a typical transfer */
#define EDT_DMA_LENGTH (512 * 1024 * 1024)

typedef enum { DMA_STOPPED, DMA_RUNNING, PROCESS_WAITING, OCCURED } intr_stat_t;

typedef struct {
  OS_SPECIFIC_RBUF os;

  u_int m_XferSize; /* bytes in buffer */

  u_int SgEntries; /* how many entries in the list of sg lists*/

  /* the old SgTodo.m_List. This is a list of loadable scatter-gather chunks
   * (i.e not crossing 64K boundaries) */
  EdtPCISg* SgChunksList;

  u_int m_nChunks; /* number of loadable SgChunksList chunks*/
  u_int InitialChunks;

  u_int AllocatedEntries;       /* How many chumks in AllocatedSgList*/
  EdtMemRange* AllocatedSgList; /* Sglists as allocated  - win has 1 element
                                                      Linux has several */

  u_int m_SglistSize; /* Length of sglist in bytes */
  u_int m_SgUsed;     /* size in bytes of sg list */

  u_int m_Idx; /* index of next SgChunksList element to load*/

  user_addr_t m_user_address;

  edt_timespec_t m_start_timeval;
  edt_timespec_t m_end_timeval;

  u_int m_WriteFlag;

  u_int m_FirstAddr; /* store for odd check */

  u_int m_LinesXferred; /* store # of lines in a
                                            shortened transfer */

  u_int* m_SgVirtAddr; /* virtual sglist addr */

  /* keep track of modifications to SG list */

  u_int is_adjusted; /* for adjusting size w/o rebuilding */

  u_int m_SgAdjustedEntry; /* If adjusted, which page has
                      adjustment */

  u_int m_SgAdjustedSize; /* If adjusted, what was the previous
                     size in that mem page*/

  u_int SgArrayAdjustedChunk; /* If adjusted, which SgChunksList chunk
                         was adjusted */

  u_int SgArrayAdjustedSize; /* If adjusted, what was original
                        SgChunksList size */

  u_int m_Active; /* */

  /* a list of sglists - each < 64k */
  u_int AllocatedChunks;

  u_int SgAllocSize;

} rbuf;

#define EDT_MAXSIGS 10

typedef struct {
  /* the following are for serial */

  int m_Wr_producer;
  int m_Wr_consumer;
  u_short m_Wr_buf[PDV_SSIZE];
  int m_Wr_count;

} edt_serial;

typedef struct edtstr {
  OS_SPECIFIC_DEV os;

  unsigned char m_use_buffer_copy;
  unsigned char m_use_kernel_buffers;

  /* Number of DMA channels this device supports. */
  int m_chancnt;

  unsigned short m_event_flags[EDT_MAX_KERNEL_EVENTS];
  char m_event_disable[EDT_MAX_KERNEL_EVENTS];

#if defined(__linux)
  /* (kernel) Virtual memory base address of memory-mapped BAR0 */
  __iomem void* bar0_mmap;
#elif defined(_WIN32)
  PVOID bar0_mmap;
#endif
  size_t bar0_size;

#if defined(__linux)
  /* (kernel) Virtual memory base address of memory-mapped BAR1 */
  __iomem void* bar1_mmap;
#elif defined(_WIN32)
  PVOID bar1_mmap;
#endif
  size_t bar1_size;

  u_int m_DmaChannel;
  u_int devid;

  pid_t dma_pid;

  rbuf* m_CurSg;
  u_int m_timeout;
  /* Number of DMA transfers completed. */
  u_int m_xfers;
  u_int m_timecount;

  u_int m_ReadTimeout;
  u_int m_WriteTimeout;
  u_int m_MaxRbufs;
#ifdef DYNAMIC_RBUFS
  rbuf* m_Rbufs;
#else
  rbuf m_Rbufs[MAX_RBUFS];
#endif

  int m_DebugLevel; /* debug level */
  int m_TraceLevel;

  /* Defined printing mask for each of the debug levels */
  u_int DebugMask[EDT_DEBUG_LEVELS + 1];

  u_int m_Unitno;
  u_int m_Numbufs;
  u_int m_ReportedNumbufs;
  u_int m_WaitIrp;

  enum edt_dma_wait_status m_WaitStatus;
  u_int m_TimeoutOK;
  u_int m_Ints;
  u_int m_Ioctls;
  uint32_t m_Todo;
  uint32_t m_Started;
  uint32_t m_Wait;
  uint32_t m_Done;
  uint32_t m_Loaded;
  uint32_t m_Prepared;
  u_int m_Curbuf;
  u_int m_Nxtbuf;

  u_int m_WriteFlag;
  u_int m_Abort;

  u_int m_TraceIdx;
  u_int m_TraceEntrys;
  u_int* m_TraceBuf;
  u_int m_TraceBufSize;
  u_char m_Dependent[EDT_DEPSIZE];
  u_char m_IoctlBuf[EDT_DEPSIZE];
  u_int m_DoStartDma;
  u_int m_DoFlush;
  u_int m_StartDmaDesc;
  u_int m_StartDmaVal;
  u_int m_DoEndDma;
  u_int m_EndDmaDesc;
  u_int m_EndDmaVal;

  u_int m_DmaActive;
  u_int m_SaveDpath;
  u_int m_Continuous;
  u_int m_LastDone;
  u_int m_LastTodo;
  u_int m_ContState;
  u_int m_DepSet;
  u_char m_DmaIntCfg;  /* store DMA_INTCFG_ONLY to minimize access */
  u_char m_DmaIntMask; /* store DMA_INTCFG_ONLY to minimize access */
  u_int m_LastCmd;     /* last value of SG_NXT_CNT read in interrupt */

  u_int m_had_timeout;
  u_int m_TimeoutAction;   /* what to do at timeout */
  u_int m_TimeoutGoodBits; /* for ssd - how many bits valid */
  u_int m_Overflow;
  u_int m_AutoDir;     /* set direction on read/write */
  u_int m_FirstFlush;  /* when to do flush */
  u_int m_StartAction; /* when to do StartDma action */
  u_int m_EndAction;   /* when to do EndDma action */

  u_int m_hadcnt;
  u_int m_CurInfo;
  uint32_t m_PciData[16]; /* 64-byte PCI configuration space data */
  u_int m_savestat;
  u_int m_Freerun;
  u_int m_UseBurstEn;
  u_int m_EventMode; /* 0 = ONCE, 1 = CONTINUOUS, 2 = SERIALIZE */
  u_int m_EventCounters[EDT_MAX_KERNEL_EVENTS];
  u_int m_EventWaitCount[EDT_MAX_KERNEL_EVENTS];
  u_int m_EventRefs[EDT_MAX_KERNEL_EVENTS];

  struct edtstr* chan_p[MAX_MINORS];

  /* pointer to serial structs */
  edt_serial* m_serial;

  u_int m_needs_wakeup;

  struct edtstr* m_chan0_p;

  /* filename of user interface xilinx bitfile, set by IOCTL */
  edt_bitpath m_Bitpath;
  /* filename of mezzanine xilinx bitfile, set by IOCTL */
  edt_bitpath m_MezzBitpath;

#if defined(PCD)
  /* Enable to set device not ready (DNR) bit in PCD_CONFIG in DMA interrupt
   * handler. Applies to CD, CDa. */
  u_int m_AbortDma_OnIntr;
  u_int m_AbortDma_OnIntr_errno;
#endif
  u_char m_DoneOnFval;
  u_char m_HasFvalDone;

  u_char m_MultiDone;

  u_char m_mmap_buffers;
  u_char m_read_back_xilinx;
  u_char m_process_in_isr;

  void (*m_edt_dep_init)(struct edtstr* edt_p);

  int (*m_edt_device_ioctl)(struct edtstr* edt_p,
                            u_int controlCode,
                            u_int inSize,
                            u_int outSize,
                            void* argBuf,
                            u_int* bytesReturned,
                            void* pIrp);

  int (*m_edt_DeviceInt)(struct edtstr* edt_p);
  int (*m_edt_hwctl)(int operation, void* context, struct edtstr* edt_p);

  u_int m_nInterrupts;
  u_int m_nLocks;
  u_int m_lock_array[MAX_LOCK_SRC + 1];

  u_int m_direct_reads[256];
  u_int m_direct_writes[256];
  u_int m_indirect_reads[256];
  u_int m_indirect_writes[256];
  u_int m_dma_reads[8];
  u_int m_dma_writes[8];

  u_int m_IndRegBase; /* Extended register access; see comment at start of edtreg.h.  -mcm 4/09 */

  u_char* reg_trace_mask[4];

#if defined(PCD)
  EdtMezzDescriptor mezz;
#endif

  int has_pcda_direction_bit;
  int m_IsOpen;
  uint64_t m_total_lock_time;
  uint64_t m_total_wait_time;
  uint64_t m_wait_start_time;
  uint64_t m_lock_start_time;
} Edt_Dev, *pEdt_Dev;

/* TimeoutAction Values */

u_int edt_get(Edt_Dev* edt_p, u_int reg);
void edt_set(Edt_Dev* edt_p, u_int reg, u_int val);
u_int edt_and(Edt_Dev* edt_p, u_int reg, u_int val);
u_int edt_or(Edt_Dev* edt_p, u_int reg, u_int val);
u_int edt_clear(Edt_Dev* edt_p, u_int reg, u_int val);
void edt_toggle(Edt_Dev* edt_p, u_int reg, u_int val);
void edt_untoggle(Edt_Dev* edt_p, u_int reg, u_int val);
void edtpci_printreg(Edt_Dev* edt_p, u_int reg, u_int val, u_int isread);

#if defined(PCD)
void edtdrv_set_direction(Edt_Dev* edt_p, int direction);
#endif

void edt_update_intcfg(Edt_Dev* edt_p);  /* write value of m_DmaIntCfg to board*/
void edt_refresh_intcfg(Edt_Dev* edt_p); /* get value of EDT_DMA_INTCFG_ONLY  and
                                         put in m_DmaIntCfg */

void edt_set_intcfg(Edt_Dev* edt_p, u_int cfg);
u_int edt_get_intcfg(Edt_Dev* edt_p);
void edt_set_intcfg_bits(Edt_Dev* edt_p, u_int mask);
void edt_clear_intcfg_bits(Edt_Dev* edt_p, u_int mask);
void edt_disable_interrupts(Edt_Dev* edt_p);
void edt_enable_interrupts(Edt_Dev* edt_p);

u_int edt_get_rfifo(Edt_Dev* edt_p);
void edt_set_rfifo(Edt_Dev* edt_p, int val);

void edtdrv_program_flash(Edt_Dev* edt_p, const u_char* bp, int size, int setget);

void edt_abort_dma_sleep(void);

void edt_get_hres_time(Edt_Dev* edt_p, edt_timespec_t* ts);

int edt_read_sg_list(Edt_Dev* edt_p, u_int bufnum, u_int* sglist, u_int sgsize);

int edt_set_sglist(Edt_Dev* edt_p, u_int bufnum, u_int* sglist, u_int sgsize);

void edt_reg_bit_setclear(Edt_Dev* edt_p, u_int reg, u_int mask);

void edt_reg_bit_clearset(Edt_Dev* edt_p, u_int reg, u_int mask);

void edtdrv_set_startdma(Edt_Dev* edt_p, u_int desc, u_int value);

void edtdrv_set_enddma(Edt_Dev* edt_p, u_int desc, u_int value);

u_int GetSgList(u_int iolength, int bufnum);
u_int BuildSgList(int bufnum);
int edt_SetLogSgList(Edt_Dev* edt_p, int bufnum, u_int* log_list, u_int log_entrys);
u_int edt_ReBuildSgList(Edt_Dev* edt_p, u_int bufnum, u_int* usersg);
void edt_DumpSGList(Edt_Dev* edt_p, int bufnum);
void edt_SwapSGList(Edt_Dev* edt_p, int bufnum);
int edt_ResizeSGList(Edt_Dev* edt_p, int bufnum, int newsize);
uint64_t edt_get_sginfo(Edt_Dev* edt_p, u_int bufnum, u_int desc);

void StartNext(Edt_Dev* edt_p);
u_int TransferCount(Edt_Dev* edt_p);
u_int TransferBufCount(Edt_Dev* edt_p, u_int* buf);
int TransferCountDebug(Edt_Dev* edt_p, int buf, int* foundend, int* size);

u_int GoodBits(Edt_Dev* edt_p);
void edt_SetVars(Edt_Dev* edt_p, u_int idx);
void edt_ClearEvent(Edt_Dev* edt_p);
u_int edt_WaitEvent(Edt_Dev* edt_p, u_int tmout);
void EdtTrace(Edt_Dev* edt_p, u_int val);
void EdtTraceTimeStamp(Edt_Dev* edt_p, edt_timespec_t* pTimeSpec, u_int UNUSED_PARAM(level));

uint64_t edt_drv_timestamp(void);

void edt_dep_set(Edt_Dev* edt_p, u_int reg_def, u_int val);
u_int edt_dep_get(Edt_Dev* edt_p, u_int reg_def);
void dep_init(Edt_Dev* edt_p);

void edt_timeout(void* arg_p);
void edt_timeout_intr(void* arg_p);
int edt_fifocount(Edt_Dev* edt_p);

void edt_start_critical(Edt_Dev* edt_p);
void edt_end_critical(Edt_Dev* edt_p);
void edt_start_critical_src(Edt_Dev* edt_p, int src);
void edt_end_critical_src(Edt_Dev* edt_p, int src);

#ifdef __linux__
void edt_prepare_dma_buffers(Edt_Dev* edt_p);
#endif

#ifdef PDV
void pdvdrv_serial_write(Edt_Dev* edt_p, char* char_p, int size, int flags);
int pdvdrv_serial_wait(Edt_Dev* edt_p, int mwait, int count);
u_int pdvdrv_serial_read(Edt_Dev* edt_p, char* buf, int size);
int pdvdrv_serial_init(Edt_Dev* edt_p);

void pdvdrv_send_msg(Edt_Dev* edt_p);
void pdvdrv_step_continuous(Edt_Dev* edt_p);

#endif

void edtdrv_pdv_set_continuous(Edt_Dev* edt_p, int state);

u_int EdtIoctlNt(Edt_Dev* edt_p,
                 u_int code,
                 u_int insize,
                 u_int outsize,
                 void* buf,
                 u_int* bytesreturned,
                 u_int pIrp);
/* Common code for NT */
u_int EdtIoctlCommon(Edt_Dev* edt_p,
                     u_int code,
                     u_int insize,
                     u_int outsize,
                     void* buf,
                     u_int* bytesreturned,
                     void* pIrp);

/* Hardware initialization */
int edt_InitializeBoard(Edt_Dev* edt_p);

/* starts the dma on the hardware */
u_int edt_SetupDma(Edt_Dev* edt_p);

/* finishes an sg list */
u_int edt_SetupRdy(Edt_Dev* edt_p);

/* Driver functions */

int edt_wakeup_dma(Edt_Dev* edt_p);

int edt_allocate_sglist(Edt_Dev* edt_p, u_int iolength, int bufnum);
int edt_wait_sig(Edt_Dev* edt_p);
int edt_wait_sema(Edt_Dev* edt_p);
void edt_UnmapBuffer(rbuf* p);
void edt_printf(Edt_Dev* edt_p, char* fmt, ...);
void edt_dep_init(Edt_Dev* edt_p);
int edt_device_ioctl(Edt_Dev* edt_p,
                     u_int controlCode,
                     u_int inSize,
                     u_int outSize,
                     void* argBuf,
                     u_int* bytesReturned,
                     void* pIrp);
void edt_abortdma(Edt_Dev* edt_p);
int edt_dma_chan_info(Edt_Dev* edt_p, edt_dma_info_t* dma_info);
int edt_waitdma(Edt_Dev* edt_p);
Edt_Dev* edt_DpcForIsr(Edt_Dev* edt_p);

u_int Transfer_BufCount(Edt_Dev* edt_p, u_int* cur_buffer);

#define edt_dep_set(edt_p, reg, val) edt_p->m_edt_dep_set(edt_p, reg, val)

void edt_wakeup_serial(Edt_Dev* edt_p);
int edt_wait_serial(Edt_Dev* edt_p, int msecs);

void edt_broadcast_event(Edt_Dev* edt_p, int nEventId);
int edt_wait_on_event(Edt_Dev* edt_p, int nEventId);

int edt_initialize_vars(Edt_Dev* edt_p);

int edt_get_active_channels(Edt_Dev* edt_p);

int edt_default_rbufs_from_type(int id);

int edt_set_max_rbufs(Edt_Dev* edt_p, int newmax);
void edt_set_tracebuf_size(Edt_Dev* edt_p, u_int newsize);
void edt_free_tracebuf(Edt_Dev* edt_p);
void edt_clear_tracebuf(Edt_Dev* edt_p);

void edt_dump_ring_data(Edt_Dev* edt_p);

void edt_time_reg_access(Edt_Dev* edt_p);
void edt_reg_set_trace_mask(Edt_Dev* edt_p, u_int reg_def);

int edt_safe_bufnum(Edt_Dev* edt_p, int bufnum);

void edt_disable_interrupts(Edt_Dev* edt_p);

void edt_enable_interrupts(Edt_Dev* edt_p);

u_int edt_get_rfifo(Edt_Dev* edt_p);

void edt_set_rfifo(Edt_Dev* edt_p, int val);

void edt_set_interrupt_mask(Edt_Dev* edt_p, u_int val);

u_int edt_get_interrupt_mask(Edt_Dev* edt_p);

void edt_enable_remote_interrupt(Edt_Dev* edt_p);

void edt_disable_remote_interrupt(Edt_Dev* edt_p);

void edt_enable_dma_interrupt(Edt_Dev* edt_p);

void edt_disable_dma_interrupt(Edt_Dev* edt_p);

int edt_set_sg_cnt(Edt_Dev* edt_p, u_int cmd);
int edt_set_sg_addr(Edt_Dev* edt_p, u_int cmd);

#define edt_DeviceInt(edt_p) (edt_p->m_edt_DeviceInt(edt_p))

#define edt_dep_init(edt_p) edt_p->m_edt_dep_init(edt_p)

#define edt_device_ioctl(edt_p, controlcode, insize, outsize, argBuf, bytesReturned, pIrp) \
  edt_p->m_edt_device_ioctl(edt_p, controlcode, insize, outsize, argBuf, bytesReturned, pIrp)

#define edt_hwctl(operation, context, edt_p) edt_p->m_edt_hwctl(operation, context, edt_p)

void edt_set_debug_mask(Edt_Dev* edt_p, u_int level, u_int mask);

u_int edt_get_debug_mask(Edt_Dev* edt_p, u_int dbglevel);

int edt_unmap_buffer(Edt_Dev* edt_p, u_int bufnum);

int edt_map_buffer(Edt_Dev* edt_p,
                   int bufnum,
                   user_addr_t user_address,
                   u_int buflength,
                   u_int direction,
                   void* pFile);

void edt_clear_sglist(Edt_Dev* edt_p, int bufnum);

void edt_lock_abort(Edt_Dev* edt_p);
void edt_unlock_abort(Edt_Dev* edt_p);

int edt_start_buffers(Edt_Dev* edt_p, u_int numtostart);

void edt_set_dma_owner(Edt_Dev* edt_p, void* pFile);

void edt_release_dma_ownership(Edt_Dev* edt_p);

int edt_can_take_dma_ownership(Edt_Dev* edt_p, void* pFile);

int edt_is_dma_owner(Edt_Dev* edt_p, void* pFile);

int edt_stop_buffer(Edt_Dev* edt_p, u_int finish_current);

int edt_init_board_specific_hw(Edt_Dev* edt_p);

void edt_dma_cleanup(Edt_Dev* edt_p, void* pFile);

void* edt_alloc_nonpaged(Edt_Dev* edt_p, size_t size, char* tag);

void* edt_alloc_paged(Edt_Dev* edt_p, size_t size, char* tag);

void edt_free_kernelmem(Edt_Dev* edt_p, void* pointer, size_t size, char* tag);

void edt_reg_set_trace_mask(Edt_Dev* edt_p, u_int reg_def);

int edt_pci_config_set(Edt_Dev* edt_p, u_int desc, u_int value);

u_int edt_pci_config_get(Edt_Dev* edt_p, u_int desc);

#ifdef _WIN32
/*!
 * @brief Map PCI BAR 0 or 1 into user space.
 *
 * Called by `ES_MAPMEM` ioctl. Use @ref edtdrv_unmapmem() to release this memory
 * mapping.
 *
 * @warning No support for mapping scatter-gather lists or DMA buffers, like
 * edt_mmap_vmap_address() does in the Linux driver.
 *
 * @param edt_p
 *   Open kernel device handle.
 * @param offset
 *   Requested address offset.
 *   @arg <= 0x10000 -- BAR 0
 *   @arg > 0x10000 -- BAR 1
 * @param length
 *   Requested memory map size. An error will be returned if
 *   @p offset + @p length exceed the BAR region.
 *
 * @return NULL on error. Userspace virtual address of mapped BAR region on success.
 */
void* edtdrv_mapmem(Edt_Dev* edt_p, uint32_t offset, uint32_t length);

/*!
 * @warning NOT IMPLEMENTED
 *
 * @brief Unmap PCI BAR regions previously setup with @ref edtdrv_mapmem().
 *
 * Will search the internal MDL list for an MDL matching the given @p address and
 * @p length. If found, the mapping will be released and the MDL free'd.
 *
 * @param edt_p
 *   Open kernel device handle.
 * @param address
 *   Userspace virtual address of the mapped region.
 * @param length
 *   Size of the mapped region.
 *
 * @retval 0 on success
 * @retval STATUS_NOT_MAPPED_VIEW if @p address and @p length did not match an
 *   MDL in the list.
 */
NTSTATUS edtdrv_unmapmem(Edt_Dev* edt_p, void* address, uint32_t length);
#endif

int edt_set_ringbuf_physical_addr(Edt_Dev* edt_p, u_int bufnum, uint64_t phyaddr);

int edt_resize_sglist(Edt_Dev* edt_p, u_int bufnum, u_int newsize, int writeflag);

int edt_set_sglist(Edt_Dev* edt_p, u_int bufnum, u_int* new_list, u_int new_entrys);

int edt_wait_event_id_user(Edt_Dev* edt_p, int eventid, int timeout);

void* edt_vmalloc(u_int size, char* tag);

void edt_vfree_sized(void* p, u_int size);

void edt_vfree(void* p, u_int size, char* tag);

void edt_set_current_buffer(Edt_Dev* edt_p, u_int bufnum);

u_int edt_set_kernel_buffers(Edt_Dev* edt_p, u_int newstate);

int TransferCountCommon(Edt_Dev* edt_p,
                        int buf,
                        u_int* endptr,
                        u_int* sizeptr,
                        int dsbldma,
                        int onlyactive);

int edt_trylock_registers(Edt_Dev* edt_p);

int edt_lock_registers(Edt_Dev* edt_p);
int edt_unlock_registers(Edt_Dev* edt_p);

/*edt_hwctl() operations and caller identity defines */
#define INIT_DEVICE      0x1
#define IDLE_DEVICE      0x2
#define START_DMA        0x3
#define ABORT_DMA        0x4
#define MMAP_REGISTERS   0x7
#define UNMMAP_REGISTERS 0x8
#define ENABLE_INTR      0x9
#define DISABLE_INTR     0xa
#define TEST_INTR        0xb
#define ACK_INTR         0xc
#define GET_DMA_RESID    0xe
#define POSTIO           0x12
#define STARTIO          0x13
#define PRE_STARTIO_OP   0x14
#define LOAD_REGISTERS   0x15
#define DMA_COUNTDOWN    0x17
#define EDT_DMA_ADDRESS  0x18
#define FIRST_OPEN       0x1b
#define LAST_CLOSE       0x1c
#define INIT_SOFTWARE    0x1d
#define FREE_SOFTWARE    0x1e

#ifndef _NT_DRIVER_
#define STATUS_SUCCESS 0
#define NTSTATUS       u_int
#define NT_SUCCESS(x)  (x == STATUS_SUCCESS)
#endif

#ifdef __cplusplus
}
#endif

#endif /* _EDT_KERNEL_STRUCT_H_ */
