/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 * @brief Local include files and declarations for Edt_Dev driver.
 *
 * Common shared driver code with hooks to map driver dependencies into driver
 * independent code.
 */
#ifndef EDT_COMMON_H
#define EDT_COMMON_H

#ifdef __cplusplus
extern "C" {
#endif

#include "edtdrv.h"

/*!
 * Determine if the interrupt is ours, clear interrupt enables.
 *
 * @param edt_p
 *     EDT dev pointer.
 *
 * @retval 0 if PCI interrupts are disabled (EDT_ICFG_PCI_EN_INTR bit is clear)
 *     or if none of the interupt status bits ( @ref EDT_DMA_INTR,
 *     @ref EDT_RMT_INTR, @ref EDT_PCI_INTR) are set.
 * @retval 1 if interrupts are enabled and an interrupt was detected.
 */
int edt_test_intr(Edt_Dev* edt_p);

#ifdef __cplusplus
}
#endif

#endif /* EDT_COMMON_H */
