/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @internal
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 */
#include "edt_driver_ioctl.h"

#include "edtdrv.h"
#include "edt_driver_version.h"

struct {
  u_int code;
  char* name;
} edt_ioctl_names[] = {{ES_DEBUG, "ES_DEBUG"},
                       {EG_DEBUG, "EG_DEBUG"},
                       {ES_REG, "ES_REG"},
                       {EG_REG, "EG_REG"},
                       {ES_SERIAL, "ES_SERIAL"},
                       {EG_SERIAL, "EG_SERIAL"},
                       {ES_DEPENDENT, "ES_DEPENDENT"},
                       {EG_DEPENDENT, "EG_DEPENDENT"},
                       {EG_DEVID, "EG_DEVID"},
                       {ES_RTIMEOUT, "ES_RTIMEOUT"},
                       {ES_WTIMEOUT, "ES_WTIMEOUT"},
                       {EG_BUFDONE, "EG_BUFDONE"},
                       {ES_NUMBUFS, "ES_NUMBUFS"},
                       {ES_BUF, "ES_BUF"},
                       {ES_STARTBUF, "ES_STARTBUF"},
#if defined(__linux__)
                       {ES_WAITBUF, "ES_WAITBUF"},
#endif
                       {ES_FREEBUF, "ES_FREEBUF"},
                       {ES_STOPBUF, "ES_STOPBUF"},
                       {ES_SETBUF, "ES_SETBUF"},
                       {EG_TIMEOUTS, "EG_TIMEOUTS"},
                       {EG_TRACEBUF, "EG_TRACEBUF"},
                       {ES_STARTDMA, "ES_STARTDMA"},
                       {ES_ENDDMA, "ES_ENDDMA"},
                       {EG_RTIMEOUT, "EG_RTIMEOUT"},
                       {EG_WTIMEOUT, "EG_WTIMEOUT"},

                       {ES_SERIALWAIT, "ES_SERIALWAIT"},
                       {EG_OVERFLOW, "EG_OVERFLOW"},
                       {ES_AUTODIR, "ES_AUTODIR"},
                       {ES_FIRSTFLUSH, "ES_FIRSTFLUSH"},
                       {ES_CONFIG, "ES_CONFIG"},
                       {EG_CONFIG, "EG_CONFIG"},
                       {EG_SGTODO, "EG_SGTODO"},
                       {EG_SGLIST, "EG_SGLIST"},
                       {ES_SGLIST, "ES_SGLIST"},
                       {EG_SGINFO, "EG_SGINFO"},
                       {EG_TIMECOUNT, "EG_TIMECOUNT"},
                       {ES_STARTACT, "ES_STARTACT"},
                       {ES_ENDACT, "ES_ENDACT"},
                       {ES_RESETCOUNT, "ES_RESETCOUNT"},
                       {ES_RESETSERIAL, "ES_RESETSERIAL"},
                       {ES_CLR_EVENT, "ES_CLR_EVENT"},
                       {ES_ADD_EVENT_FUNC, "ES_ADD_EVENT_FUNC"},
                       {ES_DEL_EVENT_FUNC, "ES_DEL_EVENT_FUNC"},
                       {ES_WAIT_EVENT, "ES_WAIT_EVENT"},
                       {EG_TMSTAMP, "EG_TMSTAMP"},
                       {ES_TIMEOUT_ACTION, "ES_TIMEOUT_ACTION"},
                       {EG_TIMEOUT_GOODBITS, "EG_TIMEOUT_GOODBITS"},
                       {ES_REG_OR, "ES_REG_OR"},
                       {ES_REG_AND, "ES_REG_AND"},
                       {EG_GOODBITS, "EG_GOODBITS"},
                       {ES_BURST_EN, "ES_BURST_EN"},
                       {EG_BURST_EN, "EG_BURST_EN"},

                       {ES_DOTIMEOUT, "ES_DOTIMEOUT"},
                       {ES_REFTMSTAMP, "ES_REFTMSTAMP"},
                       {EG_BUFBYTECOUNT, "EG_BUFBYTECOUNT"},
#if defined(PDV) && (PDV != 0)
                       {ES_PDVCONT, "ES_PDVCONT"},
                       {ES_PDVDPATH, "ES_PDVDPATH"},
#endif
                       {ES_RESET_EVENT_COUNTER, "ES_RESET_EVENT_COUNTER"},

                       {EG_TODO, "EG_TODO"},
                       {ES_RESUME, "ES_RESUME"},
                       {ES_EVENT_HNDL, "ES_EVENT_HNDL"},
                       {ES_MAX_BUFFERS, "ES_MAX_BUFFERS"},
                       {EG_MAX_BUFFERS, "EG_MAX_BUFFERS"},
                       {ES_PROG_XILINX, "ES_PROG_XILINX"},
#ifdef _NT_DRIVER_
                       {ES_MAPMEM, "ES_MAPMEM"},
#endif
                       {ES_DRV_BUFFER, "ES_DRV_BUFFER"},

                       {ES_BITPATH, "ES_BITPATH"},
                       {EG_BITPATH, "EG_BITPATH"},
                       {EG_VERSION, "EG_VERSION"},
                       {EG_BUILDID, "EG_BUILDID"},

                       {EG_MEMSIZE, "EG_MEMSIZE"},
                       {ES_DEBUG_MASK, "ES_DEBUG_MASK"},
                       {EG_DEBUG_MASK, "EG_DEBUG_MASK"},
#if defined(PCD)
                       {ES_ABORTDMA_ONINTR, "ES_ABORTDMA_ONINTR"},
#endif

#ifdef PDV
                       {ES_FVAL_DONE, "ES_FVAL_DONE"},
                       {EG_FVAL_DONE, "EG_FVAL_DONE"},
                       {EG_LINES_XFERRED, "EG_LINES_XFERRED"},
#endif
                       {ES_PROCESS_ISR, "ES_PROCESS_ISR"},
                       {ES_CLEAR_DMAID, "ES_CLEAR_DMAID"},
#ifdef PDV
                       {EG_SERIAL_WRITE_AVAIL, "EG_SERIAL_WRITE_AVAIL"},
#endif

                       {EG_WAIT_STATUS, "EG_WAIT_STATUS"},
                       {ES_WAIT_STATUS, "ES_WAIT_STATUS"},
                       {ES_TIMEOUT_OK, "ES_TIMEOUT_OK"},
                       {EG_TIMEOUT_OK, "EG_TIMEOUT_OK"},
                       {ES_MULTI_DONE, "ES_MULTI_DONE"},
                       {EG_MULTI_DONE, "EG_MULTI_DONE"},
                       {ES_TEST_LOCK_ON, "ES_TEST_LOCK_ON"},
                       {0, 0}};

static char* get_ioctl_name(unsigned int code)
{
  int i = 0;
  while (edt_ioctl_names[i].name)
  {
    if (code == edt_ioctl_names[i].code)
    {
      return edt_ioctl_names[i].name;
    }

    i++;
  }
  return "<unknown>";
}

void edt_SetVars(Edt_Dev* edt_p, u_int idx)
{
  if (edt_p->m_Numbufs == 0)
  {
    idx = 0;
  }
  else
  {
    edt_p->m_Nxtbuf = edt_safe_bufnum(edt_p, idx);
  }
  edt_p->m_Done = idx;
  edt_p->m_Freerun = 0;
  edt_p->m_Loaded = idx;
  edt_p->m_Started = idx;
  edt_p->m_Todo = idx;
  edt_p->m_Prepared = idx;
  edt_p->m_Wait = idx;
  edt_p->m_Curbuf = edt_p->m_Nxtbuf;
}

/*
 * EdtIoctlNt
 */

/* Deal with incompatible edt_buf and buf_args structures */
/* fortunately size changed between versions */

typedef struct _edt_buf_32 {
  uint32_t desc;
  uint32_t value;
} edt_buf_32;

typedef struct _edt_buf_nopad {
  uint64_t value;
  uint32_t desc;
} edt_buf_nopad;

static int get_ioctl_edt_buf(Edt_Dev* edt_p, void* argbuf, u_int inSize, edt_buf** outbuf_ptr)
{
  /* always use ptr if 64-bit Linux */
#if defined(__linux__) && defined(CONFIG_COMPAT)
  (void)edt_p;  /* unused */
  (void)inSize; /* unused */

  *outbuf_ptr = (edt_buf*)argbuf;
#else
  if (inSize == sizeof(edt_buf))
  {
    *outbuf_ptr = (edt_buf*)argbuf;
  }
  else if (inSize == sizeof(edt_buf_32))
  {
    edt_buf_32* buf32 = (edt_buf_32*)argbuf;
    edt_buf* outbuf = *outbuf_ptr;
    outbuf->desc = buf32->desc;
    outbuf->value = buf32->value;
  }
  else if (inSize == sizeof(edt_buf_nopad))
  {
    edt_buf_nopad* buf_nopad = (edt_buf_nopad*)argbuf;
    edt_buf* outbuf = *outbuf_ptr;
    outbuf->desc = buf_nopad->desc;
    outbuf->value = buf_nopad->value;
  }
  else if (inSize != 0)
  {
    EDTPRT(edt_p, DBG_IOCTLS, TRL_ERROR, ("Size %d not valid for edt_buf\n", inSize));
    return -1;
  }
#endif /* always use ptr */

  return 0;
}

static int put_ioctl_edt_buf(Edt_Dev* edt_p, void* argbuf, u_int inSize, edt_buf** outbuf_ptr)
{
  /* always use ptr if 64-bit Linux */
#if defined(__linux__) && defined(CONFIG_COMPAT)
  (void)edt_p;  /* unused */
  (void)inSize; /* unused */
  *outbuf_ptr = (edt_buf*)argbuf;
#else
  if (inSize == sizeof(edt_buf))
  {
    *outbuf_ptr = (edt_buf*)argbuf;
  }
  else if (inSize == sizeof(edt_buf_32))
  {
    edt_buf_32* buf32 = (edt_buf_32*)argbuf;
    edt_buf* outbuf = *outbuf_ptr;
    buf32->desc = outbuf->desc;
    buf32->value = (u_int)outbuf->value;
  }
  else if (inSize == sizeof(edt_buf_nopad))
  {
    edt_buf_nopad* buf_nopad = (edt_buf_nopad*)argbuf;
    edt_buf* outbuf = *outbuf_ptr;
    buf_nopad->desc = outbuf->desc;
    buf_nopad->value = outbuf->value;
  }
  else if (inSize != 0)
  {
    EDTPRT(edt_p, DBG_IOCTLS, TRL_ERROR, ("Size %d not valid for edt_buf\n", inSize));
    return -1;
  }
#endif /* always use ptr */

  return 0;
}

typedef struct buf_args32 {
  u_int index;
  u_int addr;
  u_int size;
  u_int writeflag;
} buf_args_32;

static int get_ioctl_buf_args(Edt_Dev* edt_p, void* argbuf, u_int inSize, buf_args** outbuf_ptr)
{
  /* always use ptr if 64-bit Linux */
#if defined(__linux__)

#if IS_ENABLED(CONFIG_COMPAT)
  (void)edt_p;  /* unused */
  (void)inSize; /* unused */
  *outbuf_ptr = (buf_args*)argbuf;
#endif /* IS_ENABLED */

#else
  buf_args* outbuf;

  if (inSize == sizeof(buf_args))
  {
    *outbuf_ptr = (buf_args*)argbuf;
  }
  else if (inSize == sizeof(buf_args_32))
  {
    buf_args_32* buf32 = (buf_args_32*)argbuf;
    outbuf = *outbuf_ptr;
    outbuf->index = buf32->index;
    outbuf->addr = buf32->addr;
    outbuf->size = buf32->size;
    outbuf->writeflag = buf32->writeflag;
  }
  else if (inSize != 0)
  {
    EDTPRT(edt_p,
           DBG_IOCTLS,
           TRL_ERROR,
           ("inSize %d not valid for buf_args: buf_args %d buf_args32 %d\n",
            inSize,
            sizeof(buf_args),
            sizeof(buf_args_32)));
    return -1;
  }
#endif /* always use ptr */

  return 0;
}

static int put_ioctl_buf_args(Edt_Dev* edt_p, void* argbuf, u_int outSize, buf_args** outbuf_ptr)
{
  /* always use ptr if 64-bit Linux */
#if defined(__linux__) && defined(CONFIG_COMPAT)
  (void)edt_p;   /* unused */
  (void)outSize; /* unused */
  *outbuf_ptr = (buf_args*)argbuf;
#else
  if (outSize == sizeof(buf_args))
  {
    *outbuf_ptr = (buf_args*)argbuf;
  }
  else if (outSize == sizeof(buf_args_32))
  {
    buf_args_32* buf32 = (buf_args_32*)argbuf;
    buf_args* outbuf = *outbuf_ptr;
    buf32->index = outbuf->index;
    buf32->addr = (u_int)outbuf->addr;
    buf32->size = outbuf->size;
    buf32->writeflag = outbuf->writeflag;
  }
  else if (outSize != 0)
  {
    EDTPRT(edt_p, DBG_IOCTLS, TRL_ERROR, ("outSize %d not valid for buf_args\n", outSize));
    return -1;
  }
#endif /* always use ptr */

  return 0;
}

#define GET_EDT_BUF(edt_p, ep)                                 \
  if (sizeof(edt_buf) == inSize || sizeof(edt_buf) == outSize) \
    ep = (edt_buf*)argBuf;                                     \
  else if (get_ioctl_edt_buf(edt_p, argBuf, inSize, &ep))      \
    return EINVAL;

#define PUT_EDT_BUF(edt_p, ep)               \
  if (outSize && sizeof(edt_buf) != outSize) \
    put_ioctl_edt_buf(edt_p, argBuf, outSize, &ep);

#define GET_BUF_ARGS(edt_p, bp)                                  \
  if (sizeof(buf_args) == inSize || sizeof(buf_args) == outSize) \
    bp = (buf_args*)argBuf;                                      \
  else if (get_ioctl_buf_args(edt_p, argBuf, inSize, &bp))       \
    return EINVAL;

#define PUT_BUF_ARGS(edt_p, bp)              \
  if (outSize && sizeof(edt_buf) != outSize) \
    put_ioctl_buf_args(edt_p, argBuf, outSize, &bp);

u_int EdtIoctlCommon(Edt_Dev* edt_p,
                     u_int controlCode,
                     u_int inSize,
                     u_int outSize,
                     void* argBuf,
                     u_int* bytesReturned,
                     void* pIrp)
{
  u_char arg8 = 0;
  u_short arg16 = 0;
  u_int arg32 = 0;

  int retval = 0;
  edt_buf ebuf;
  buf_args bargs;
  edt_buf* ep = &ebuf;
  buf_args* bp = &bargs;

  u_int tmp;

  /* for actions that apply to the whole board like bitload */

  Edt_Dev* dev0_p;

  if (edt_p->m_chan0_p)
    dev0_p = edt_p->m_chan0_p;
  else
    dev0_p = edt_p;

  if (argBuf == 0 && (inSize || outSize))
  {
    EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("code %x in argBuf 0\n", controlCode));
  }
  else
  {
    switch (inSize)
    {
    case 1:
      arg8 = *(u_char*)argBuf;
      arg16 = arg8;
      arg32 = arg8;
      break;
    case 2:
      arg16 = *(u_short*)argBuf;
      arg32 = arg16;
      break;
    case 4:
      arg32 = *(u_int*)argBuf;
      break;
    default:
      break;
    }
  }

  EDTPRT(edt_p,
         DBG_IOCTLS,
         TRL_VERBOSE,
         ("ioctl size %d code %x name %s arg %d\n",
          inSize,
          controlCode,
          get_ioctl_name(controlCode),
          arg32));

  if (controlCode == ES_SERIAL)
    EDTPRT(edt_p,
           DBG_IOCTLS,
           TRL_VERBOSE,
           ("ES_SERIAL: DMACFG %08x\n", edt_get(edt_p, EDT_DMA_CFG)));

  edt_p->m_Ioctls++;

  *bytesReturned = outSize;

  switch (controlCode)
  {
  case ES_DEBUG:

    edt_p->m_DebugLevel = arg32 & 0xffff;
    /* Clamp to 0 to 10, invert ?*/
    edt_set_debug_mask(edt_p, edt_p->m_DebugLevel, DBG_ALL & ~(DBG_REG | DBG_LOCKS));

    edt_p->m_TraceLevel = arg32 >> 16;

    break;

  case ES_TRACE_REG:
    edt_reg_set_trace_mask(edt_p, arg32);
    break;

  case EG_DEBUG:
    arg32 = edt_p->m_DebugLevel;
    break;

  case ES_DEBUG_MASK:
    /* Specify a mask and a debug level */
    /* all mask values that match are set to that level */

    {
      u_int maskval = DBG_MASK_VAL(arg32);
      u_int dbgval = DBG_MASK_LEVEL(arg32);

      edt_set_debug_mask(edt_p, dbgval, maskval);
    }

    break;

  case EG_DEBUG_MASK:
  {
    /* Returns the mask for a particular level */
    u_int dbgval = DBG_MASK_LEVEL(arg32);

    arg32 = edt_get_debug_mask(edt_p, dbgval);
  }

  break;

  case ES_TEST_LOCK_ON:
#ifdef __linux__

    if (arg32)
      edt_lock_registers(edt_p);
    else
      edt_unlock_registers(edt_p);
#endif

    break;

#ifdef _WIN32
  case ES_MAPMEM:
    GET_EDT_BUF(edt_p, ep);
    ep->value = (uint64_t)edtdrv_mapmem(edt_p, (uint32_t)ep->value, ep->desc);
    PUT_EDT_BUF(edt_p, ep);
    break;

  case ES_UNMAPMEM:
    GET_EDT_BUF(edt_p, ep);
    retval = edtdrv_unmapmem(edt_p, (void*)ep->value, ep->desc);
    break;
#endif

  case EG_MEMSIZE:
    arg32 = (uint32_t)edt_p->bar0_size;
    break;

  case EG_MEM2SIZE:
    arg32 = (uint32_t)edt_p->bar1_size;
    break;

  case ES_DRV_BUFFER:
  {
#if defined(__linux__) || defined(WDF_DRIVER)

    arg32 = edt_set_kernel_buffers(edt_p, arg32);

#else
    arg32 = 0;
#endif
  }
  break;

  case EG_CONFIG_COPY:
    GET_EDT_BUF(edt_p, ep);
    EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("getting config %llu bytes off %d", ep->value, ep->desc));
    edt_memcpy(argBuf, edt_p->m_PciData + ep->desc, (u_int)ep->value);
    *bytesReturned = (u_int)ep->value;
    PUT_EDT_BUF(edt_p, ep);

    /*
     * prevent argBuf from getting clobbered at the bottom if outSize is
     * 1, 2 or 4
     */
    outSize = 0;
    break;

  case ES_CONFIG:
    GET_EDT_BUF(edt_p, ep);
#ifdef _NT_DRIVER_
    edt_pci_config_set(edt_p, ep->desc, (u_int)ep->value);
#elif defined(__linux__)
    retval = pci_write_config_dword(edt_p->os.m_pci_dev, ep->desc, (u_int)ep->value);
    retval = pcibios_err_to_errno(retval);
#endif
    PUT_EDT_BUF(edt_p, ep);

    break;

  case EG_CONFIG:
    GET_EDT_BUF(edt_p, ep);
#ifdef _NT_DRIVER_
    ep->value = edt_pci_config_get(edt_p, ep->desc);
#elif defined(__linux__)
    {
      uint32_t value;
      retval = pci_read_config_dword(edt_p->os.m_pci_dev, ep->desc, &value);
      ep->value = value;
      retval = pcibios_err_to_errno(retval);
    }
#endif
    PUT_EDT_BUF(edt_p, ep);

    break;

  case EG_SGINFO:
    GET_EDT_BUF(edt_p, ep);
    ep->value = edt_get_sginfo(edt_p, (u_int)ep->value, ep->desc);
    PUT_EDT_BUF(edt_p, ep);
    break;

  case EG_SGTODO:
  {
    rbuf* p = &edt_p->m_Rbufs[edt_p->m_CurInfo];
    if (p == 0 || p->m_nChunks == 0)
    {
      *bytesReturned = 0;
      break;
    }

    tmp = p->m_nChunks * sizeof(EdtMemRange);
    if (tmp > outSize)
      tmp = outSize;

    if (tmp)
    {
#ifdef _WINDK_DRIVER_
      edt_memcpy(argBuf, p->m_SgTodo.m_List, outSize);
#else
      edt_memcpy(argBuf, p->SgChunksList, outSize);
#endif
      *bytesReturned = outSize;

      /*
       * prevent argBuf from getting clobbered at the bottom if outSize is
       * 1, 2 or 4 (pretty unlikely with Dependent anyway)
       */
      outSize = 0;
    }
  }
  break;

  case EG_SGLIST:
  {
    u_int bufnum;
    u_int* sg_uaddr;
    rbuf* p = NULL;
#if !defined(__linux__)
    u_int* sg_list;
    u_int sg_size;
#endif

    GET_BUF_ARGS(edt_p, bp);
    bufnum = bp->index;
    p = &edt_p->m_Rbufs[bufnum];
    if (p == 0 || p->m_SglistSize == 0)
    {
      EDTPRT(edt_p, DBG_IOCTLS, TRL_ERROR, ("zero sglist in get\n"));
      *bytesReturned = 0;
      break;
    }

    sg_uaddr = (u_int*)(bp->addr);

#if defined(__linux__)
    edt_read_sg_list(edt_p, bp->index, sg_uaddr, bp->size);
#else

    sg_list = (u_int*)p->m_SgVirtAddr;

    sg_size = bp->size;
    userout(sg_list, sg_uaddr, sg_size);
#endif
    PUT_BUF_ARGS(edt_p, bp);
  }
  break;

  case ES_SGLIST:
  {
    u_int* log_list;
    u_int log_entrys;
    u_int bufnum;
    u_int* log_uaddr;
    GET_BUF_ARGS(edt_p, bp)
    bufnum = bp->index;
    log_entrys = bp->size;
    log_uaddr = (u_int*)((long)bp->addr);
#if defined(__linux__)
    log_list = (u_int*)edt_vmalloc(log_entrys * 8, "log_list");
#elif defined(_NT_DRIVER_)
    log_list = (u_int*)edt_vmalloc(log_entrys * 8, "log_list");
#endif
    userin(log_uaddr, log_list, log_entrys * 8);
    (void)edt_SetLogSgList(edt_p, bufnum, log_list, log_entrys);
#if defined(__linux__)
    edt_vfree(log_list, log_entrys * 8, "log_list");
#elif defined(_NT_DRIVER_)
    edt_vfree(log_list, log_entrys * 8, "log_list");
#endif
    PUT_BUF_ARGS(edt_p, bp);
    break;
  }

  /* register set	and get	 */
  case ES_REG:
    GET_EDT_BUF(edt_p, ep);
    if (edt_trylock_registers(edt_p) != 0)
    {
      EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("Set Intfc locked\n"));
      edt_lock_registers(edt_p);
    }

    edt_start_critical_src(edt_p, 1);
    edt_set(edt_p, ep->desc, (u_int)ep->value);
    edt_end_critical_src(edt_p, 1);
    edt_unlock_registers(edt_p);

    break;

  case EG_REG:
    GET_EDT_BUF(edt_p, ep);
    if (edt_trylock_registers(edt_p) != 0)
    {
      EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("Set Intfc locked\n"));
      edt_lock_registers(edt_p);
    }
    edt_start_critical_src(edt_p, 2);
    arg32 = edt_get(edt_p, ep->desc);
    ep->value = arg32;
    edt_end_critical_src(edt_p, 2);
    edt_unlock_registers(edt_p);
    PUT_EDT_BUF(edt_p, ep);

    break;

  case ES_REG_OR:
    if (edt_trylock_registers(edt_p) != 0)
    {
      EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("Set Reg OR locked\n"));
      edt_lock_registers(edt_p);
    }
    GET_EDT_BUF(edt_p, ep);
    edt_start_critical_src(edt_p, 3);
    ep->value = edt_or(edt_p, ep->desc, (u_int)ep->value);
    edt_end_critical_src(edt_p, 3);
    edt_unlock_registers(edt_p);
    PUT_EDT_BUF(edt_p, ep);
    break;

  case ES_REG_AND:
    if (edt_trylock_registers(edt_p) != 0)
    {
      EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("Set Reg AND locked\n"));
      edt_lock_registers(edt_p);
    }
    GET_EDT_BUF(edt_p, ep);
    edt_start_critical_src(edt_p, 4);
    ep->value = edt_and(edt_p, ep->desc, (u_int)ep->value);
    edt_end_critical_src(edt_p, 4);
    edt_unlock_registers(edt_p);
    PUT_EDT_BUF(edt_p, ep);
    break;

  case ES_STARTDMA:
    GET_EDT_BUF(edt_p, ep);
    edtdrv_set_startdma(edt_p, ep->desc, (u_int)ep->value);
    break;

  case ES_ENDDMA:
    GET_EDT_BUF(edt_p, ep);
    edtdrv_set_enddma(edt_p, ep->desc, (u_int)ep->value);
    break;

  case ES_REG_BIT_CLEARSET:
    GET_EDT_BUF(edt_p, ep);
    edt_reg_bit_clearset(edt_p, ep->desc, (u_int)ep->value);
    break;

  case ES_REG_BIT_SETCLEAR:
    GET_EDT_BUF(edt_p, ep);
    edt_reg_bit_setclear(edt_p, ep->desc, (u_int)ep->value);
    break;

  case ES_TIMEOUT_OK:
    edt_p->m_TimeoutOK = arg32;
    break;

  case EG_TIMEOUT_OK:
    arg32 = edt_p->m_TimeoutOK;
    break;

  case ES_MULTI_DONE:
    edt_p->m_MultiDone = (char)arg32;
    break;

  case EG_MULTI_DONE:
    arg32 = edt_p->m_MultiDone;
    break;

  case EG_WAIT_STATUS:
    arg32 = edt_p->m_WaitStatus;
    break;

  case ES_WAIT_STATUS:
    edt_p->m_WaitStatus = arg32;
    break;

  case ES_BURST_EN:
    edt_p->m_UseBurstEn = arg32;
    break;

  case EG_BURST_EN:
    arg32 = edt_p->m_UseBurstEn;
    break;

#if defined(PDV) && (PDV != 0)
  case ES_PDVDPATH:
    edt_p->m_SaveDpath = arg32;
    break;

  case ES_PDVCONT:
    edtdrv_pdv_set_continuous(edt_p, arg32);
    break;
#endif

  case EG_BUFBYTECOUNT:
  {
    u_int* args = (u_int*)argBuf;
    u_int cur_buffer;
    edt_start_critical_src(edt_p, 20);

#ifdef _NT_DRIVER_
    args[0] = TransferCount(edt_p);
    cur_buffer = edt_p->m_Curbuf;
#else
    args[0] = TransferBufCount(edt_p, &cur_buffer);
#endif
    args[1] = cur_buffer;
    edt_end_critical_src(edt_p, 20);

    break;
  }

  case EG_TIMECOUNT:
    arg32 = edt_p->m_timecount;
    break;

    /* added 10/23/98 steve	 */

  case EG_TIMEOUT_GOODBITS:
    arg32 = edt_p->m_TimeoutGoodBits;
    break;

  case ES_TIMEOUT_ACTION:
    edt_p->m_TimeoutAction = arg32;
    break;

  case ES_PROG_READBACK:
    edt_p->m_read_back_xilinx = (u_char)arg32;
    break;

  case ES_PROG_XILINX:
  {
    edt_sized_buffer* sizedbuf = (edt_sized_buffer*)argBuf;
    u_char* p_data = (u_char*)sizedbuf->data;

    edtdrv_program_flash(edt_p, p_data, sizedbuf->size, edt_p->m_read_back_xilinx);
  }

  break;

  case ES_DEPENDENT:

    edt_memcpy(edt_p->m_Dependent, argBuf, inSize);
    edt_p->m_DepSet = 1;
    break;

  case EG_DEPENDENT:
    edt_memcpy(argBuf, edt_p->m_Dependent, outSize);
    *bytesReturned = outSize;

    /*
     * prevent argBuf from getting clobbered at the bottom if outSize is
     * 1, 2 or 4 (pretty unlikely with Dependent anyway)
     */
    outSize = 0;
    break;

  case EG_TRACEBUF:
    edt_p->m_TraceBuf[0] = edt_p->m_TraceEntrys; /* EdtTraceBufData.entries */
    edt_p->m_TraceBuf[1] = edt_p->m_TraceIdx;    /* EdtTraceBufData.start_index */
    edt_memcpy(argBuf, edt_p->m_TraceBuf, outSize);
    *bytesReturned = outSize;

    /*
     * prevent argBuf from getting clobbered at the bottom if outSize is
     * 1, 2 or 4 (pretty unlikely with Tracebuf anyway)
     */
    outSize = 0;
    break;

  case ES_BITPATH:
    edt_memcpy(dev0_p->m_Bitpath, argBuf, inSize);
    break;

  case EG_BITPATH:
    edt_memcpy(argBuf, dev0_p->m_Bitpath, outSize);
    *bytesReturned = outSize;
    outSize = 0;
    break;

  case ES_MEZZ_BITPATH:
    edt_memcpy(dev0_p->m_MezzBitpath, argBuf, inSize);
    break;

  case EG_MEZZ_BITPATH:
    edt_memcpy(argBuf, dev0_p->m_MezzBitpath, outSize);

    *bytesReturned = outSize;
    outSize = 0;
    break;

  case EG_DMA_INFO:
    edt_dma_chan_info(edt_p, (edt_dma_info_t*)argBuf);
    *bytesReturned = outSize;
    outSize = 0;
    break;

  case EG_BUILDID:

#if defined(_WIN32)
    /* Same args as snprintf(), but returns STATUS_* codes instead of length. */
    retval =
        RtlStringCbPrintfA(argBuf, outSize, "%s %zu bit", EDT_DRIVER_BUILDID, (8 * sizeof(void*)));
#else
    retval = snprintf(argBuf, outSize, "%s %zu bit", EDT_DRIVER_BUILDID, (8 * sizeof(void*)));
    if (retval < 0)
      retval = -EBADE;
    else if (retval >= (int)outSize)
      retval = -EOVERFLOW;
    else
      retval = 0;
#endif

    *bytesReturned = outSize;
    outSize = 0;
    break;

  case EG_VERSION:

#if defined(_WIN32)
    retval = RtlStringCbPrintfA(argBuf, outSize, "%s", EDT_DRIVER_VERSION);
#else
    retval = snprintf(argBuf, outSize, "%s", EDT_DRIVER_VERSION);
    if (retval < 0)
      retval = -EBADE;
    else if (retval >= (int)outSize)
      retval = -EOVERFLOW;
    else
      retval = 0;
#endif

    *bytesReturned = outSize;
    outSize = 0;
    break;

  case EG_DEVID:
    arg32 = edt_p->devid;
    break;

  case ES_RTIMEOUT:
    edt_p->m_ReadTimeout = arg32;
    break;

  case ES_WTIMEOUT:
    edt_p->m_WriteTimeout = arg32;
    break;

  case EG_RTIMEOUT:
    arg32 = edt_p->m_ReadTimeout;
    break;

  case EG_WTIMEOUT:
    arg32 = edt_p->m_WriteTimeout;
    break;

#if defined(PCD)
  case ES_DIRECTION:
    edt_start_critical_src(edt_p, 50);
    edtdrv_set_direction(edt_p, arg32);
    edt_end_critical_src(edt_p, 50);
    break;
#endif /* PCD */

  case ES_FIRSTFLUSH:
    edt_p->m_FirstFlush = arg32;
    if (edt_p->m_FirstFlush == EDT_ACT_ONCE || edt_p->m_FirstFlush == EDT_ACT_ALWAYS)
    {
      edt_p->m_DoFlush = 1;
    }
    else if (edt_p->m_FirstFlush == EDT_ACT_NEVER)
      edt_p->m_DoFlush = 0;
    break;

  case ES_AUTODIR:
    edt_p->m_AutoDir = arg32;
    break;

  case EG_MAX_BUFFERS:
    arg32 = edt_p->m_MaxRbufs;
    break;

  case ES_MAX_BUFFERS:
  {
    edt_set_max_rbufs(edt_p, arg32);
  }
  break;

  case ES_NUMBUFS:
    EDTPRT(edt_p, DBG_DMA_SETUP, TRL_INFO, ("Setting numbufs to %d\n", arg32));

    edt_p->m_Numbufs = arg32;
    edt_p->m_ReportedNumbufs = arg32;
    edt_p->m_Freerun = 0;

    edt_p->m_Abort = 0;
    edt_p->m_hadcnt = 0;
    edt_p->m_ContState = 0;

#ifdef _WINDK_DRIVER_

    edt_p->ClearEvent(EDT_EVENT_BUF);

    edt_p->m_WaitIrp = NULL;
#else
    edt_p->m_WaitIrp = 0;
#endif
    edt_SetVars(edt_p, 0);
    edt_p->m_ContState = 0;

    break;

  case EG_NUMBUFS:
    arg32 = edt_p->m_Numbufs;
    break;

  case ES_SETBUF:
    edt_set_current_buffer(edt_p, arg32);
    break;

  case ES_BUF_MMAP:

    GET_BUF_ARGS(edt_p, bp)

    EDTPRT(edt_p,
           DBG_IOCTLS,
           TRL_INFO,
           ("BUF_MMAP ADDR:buf %d addr %lx size %x writeflag	%d\n",
            bp->index,
            bp->addr,
            bp->size,
            bp->writeflag));

    retval = edt_map_buffer(edt_p, bp->index, (user_addr_t)bp->addr, bp->size, bp->writeflag, pIrp);

    break;
#ifdef _WDF_DRIVER_
  case ES_ALLOC_KBUFFER:
    GET_BUF_ARGS(edt_p, bp);

    EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("ALLOC_KBUFFER ADDR: size %x \n", bp->size));

    bp->addr = (uint64_t)edt_alloc_kbuffer(edt_p, bp->size);
    EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("ALLOC_KBUFFER ADDR: addr %p\n", bp->addr));
    break;
#endif

  case ES_BUF:

    GET_BUF_ARGS(edt_p, bp);

    if (bp->addr == 0)
    {
      /* resize an existing buffer */
      edt_resize_sglist(edt_p, bp->index, bp->size, bp->writeflag);
    }
    else if (bp->size == 0)
    {
      /* Set the 32-bit physical address in the ring buffer (for direct DMA) */
      edt_set_ringbuf_physical_addr(edt_p, bp->index, bp->addr);
    }
    else
    {
      retval =
          edt_map_buffer(edt_p, bp->index, (user_addr_t)bp->addr, bp->size, bp->writeflag, pIrp);
    }
    PUT_BUF_ARGS(edt_p, bp);
    break;

  case ES_STARTBUF:

    retval = edt_start_buffers(edt_p, arg32);
    break;

  case ES_RESUME:
    edt_start_critical_src(edt_p, 7);
    edt_p->m_Abort = 0;
    StartNext(edt_p);
    edt_end_critical_src(edt_p, 7);
    break;

  case ES_DOTIMEOUT:
    edt_timeout(edt_p);
    break;

#if defined(__linux__)
  case ES_WAITBUF:
    /* if already done - dont bother with critical section */
    edt_p->m_Wait = *(uint32_t*)argBuf;

    EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("wait %x done %x\n", edt_p->m_Wait, edt_p->m_Done));

    EDTPRT(edt_p, DBG_IOCTLS, TRL_VERBOSE, ("Waiting ...\n"));
    edt_p->m_WaitIrp = 1;
    edt_waitdma(edt_p);
    edt_p->m_WaitIrp = 0;

    break;
#endif

  case ES_STOPBUF:
  {
    u_int finish_current = *(u_int*)argBuf;
    edt_start_critical_src(edt_p, 10);
    EDTPRT(edt_p,
           DBG_IOCTLS,
           TRL_INFO,
           ("Stopbuf finish %d done %d todo %d loaded %d\n",
            finish_current,
            edt_p->m_Done,
            edt_p->m_Todo,
            edt_p->m_Loaded));

    edt_p->m_Freerun = 0;

    if (finish_current)
    {
      edt_p->m_Todo = edt_p->m_Loaded;
    }
    else
    {
      edt_abortdma(edt_p);
#if defined(__linux__)
      /* DEBUG */
      EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("DEBUGG - in stopbuf - not wakeup dma\n"));
      edt_wakeup_dma(edt_p);
#endif
    }

    edt_end_critical_src(edt_p, 10);
    break;
  }

  case ES_FREEBUF:
  {
    edt_start_critical_src(edt_p, 11);

    EDTPRT(edt_p,
           DBG_IOCTLS,
           TRL_INFO,
           ("Freebuf calling abort %x %x %x %x\n",
            edt_p->m_Done,
            edt_p->m_Loaded,
            edt_p->m_Todo,
            edt_get(edt_p, EDT_SG_CUR_ADDR)));

    /*
     * DmaActive is no longer reliable.  Call abortdma consistently
     * with other calls in the current driver.  Still need to
     * check thread ID in all places.     Mark  07/13/04
     */
    edt_abortdma(edt_p);
#ifdef __linux__
    edt_abort_dma_sleep();
#endif

    edt_unmap_buffer(edt_p, arg32);

    edt_end_critical_src(edt_p, 11);
  }
  break;

  case ES_CLEAR_DMAID:
#ifdef _WINDK_DRIVER_
    PIO_STACK_LOCATION pIrpStack;  // our portion of the irp
    pIrpStack = IoGetCurrentIrpStackLocation((PIRP)pIrp);

    if (edt_p->dma_pid && (edt_p->dma_pid == pIrpStack->FileObject))
    {
      edt_p->dma_pid = 0;
    }
#elif defined(__linux__)

    edt_clear_dma(edt_p);

#endif
    break;

  case EG_BUFDONE:
    *(uint32_t*)argBuf = edt_p->m_Done;
    /*
     * prevent argBuf from getting clobbered at the bottom if outSize is
     * 1, 2 or 4
     */
    outSize = 0;
    break;

  case EG_OVERFLOW:
    arg32 = edt_p->m_Overflow;
    break;

  case EG_TIMEOUTS:
    arg32 = edt_p->m_timeout;
    break;

#ifdef PDV

  case ES_SERIAL:
  {
    ser_buf* ser_args = (ser_buf*)argBuf;

    EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("ES_SERIAL: DMACFG %08x\n", edt_get(edt_p, EDT_DMA_CFG)));

    (void)pdvdrv_serial_write(edt_p, ser_args->buf, ser_args->size, ser_args->flags);
    edt_enable_remote_interrupt(edt_p);
    break;
  }

  case EG_SERIAL:
  {
    ser_buf* ser_args = (ser_buf*)argBuf;

    edt_start_critical_src(edt_p, 13);
    *bytesReturned = pdvdrv_serial_read(edt_p, (char*)argBuf, ser_args->size);

    /*
     * prevent argBuf from getting clobbered at the bottom if outSize is
     * 1, 2 or 4
     */
    outSize = 0;

    retval = 0;
    edt_end_critical_src(edt_p, 13);
    break;
  }

  case ES_SERIALWAIT:
    GET_EDT_BUF(edt_p, ep);
    edt_start_critical_src(edt_p, 14);
    edt_enable_remote_interrupt(edt_p);
    edt_end_critical_src(edt_p, 14);
    ep->value = pdvdrv_serial_wait(edt_p, ep->desc, (int)ep->value);
    PUT_EDT_BUF(edt_p, ep);
    break;

  case EG_SERIAL_WRITE_AVAIL:
  {
    edt_serial* ser_p;
    ser_p = edt_p->m_serial;
    if (ser_p->m_Wr_count < 0)
    {
      EDTPRT(edt_p, DBG_IOCTLS, TRL_ERROR, ("ser_p-<m_Wr_count = %d\n", ser_p->m_Wr_count));
      ser_p->m_Wr_count = 0;
    }

    arg32 = PDV_SSIZE - ser_p->m_Wr_count - 1;
    if (arg32 > PDV_SSIZE)
      arg32 = PDV_SSIZE;
  }
  break;
#endif /* PDV */

  case ES_RESET_EVENT_COUNTER:
  {
    Edt_Dev* edt_ch0 = edt_p->m_chan0_p;

    edt_start_critical_src(edt_ch0, 22);
    edt_ch0->m_EventCounters[arg32] = 0;
    edt_ch0->m_EventWaitCount[arg32] = 0;
    edt_end_critical_src(edt_ch0, 22);

    break;
  }

  case ES_CLR_EVENT:
  {
    Edt_Dev* edt_ch0 = edt_p->m_chan0_p;

#ifdef _WINDK_DRIVER_
    if (edt_ch0->os.Events[arg32])
    {
      int rc;

      rc = edt_ch0->os.Events[arg32]->Reset();
      EDTPRT(edt_ch0, DBG_IOCTLS, TRL_INFO, ("Clear	Event %d state = %d\n", arg32, rc));
    }
#endif
    retval = edt_device_ioctl(edt_ch0, controlCode, inSize, outSize, argBuf, bytesReturned, pIrp);

    break;
  }

  case ES_ADD_EVENT_FUNC:
  {
    Edt_Dev* edt_ch0 = edt_p->m_chan0_p;
    /* Event mode passed in as high 8 bits.  See edt_ioctls.h */
    u_int event_type = arg32 & ~EDT_EVENT_MODE_MASK;
    edt_ch0->m_EventMode = (arg32 & EDT_EVENT_MODE_MASK) >> EDT_EVENT_MODE_SHFT;
    arg32 = event_type;
    *((u_int*)argBuf) = arg32;

    /* only	eodma is implemented as	a base event */
    /* all others are device specific */

    edt_start_critical_src(edt_ch0, 23);
    edt_ch0->m_event_disable[arg32] = 0;
    edt_ch0->m_EventRefs[arg32]++;
    edt_ch0->m_EventWaitCount[arg32] = 0;
    edt_ch0->m_EventCounters[arg32] = 0;

    edt_end_critical_src(edt_ch0, 23);

    retval = edt_device_ioctl(edt_ch0, controlCode, inSize, outSize, argBuf, bytesReturned, pIrp);
  }
  break;

  case ES_DEL_EVENT_FUNC:
  {
    Edt_Dev* edt_ch0 = edt_p->m_chan0_p;

    /* only	eodma is implemented as	a base event */
    /* all others are device specific */

    retval = edt_device_ioctl(edt_ch0, controlCode, inSize, outSize, argBuf, bytesReturned, pIrp);

    /* decrement count of references */
    /* Event is pnly active	if > 0 */

    if (edt_ch0->m_EventRefs[arg32])
    {
      /* edt_ch0->os.Events[arg32]->Set(0); */

      edt_ch0->m_event_disable[arg32] = 1;
      edt_broadcast_event(edt_ch0, arg32);
      edt_ch0->m_EventRefs[arg32]--;
    }
    else
      /* error - mismatched add/del event)func */
      EDTPRT(edt_ch0, DBG_IOCTLS, TRL_INFO, ("EventRefs[%d] already 0 on release", arg32));

    break;
  }

  case ES_WAIT_EVENT:
  {
    Edt_Dev* edt_ch0 = edt_p->m_chan0_p;

    ++edt_ch0->m_EventWaitCount[arg32];
    if (edt_ch0->m_EventWaitCount[arg32] > edt_ch0->m_EventCounters[arg32])
    {
      EdtTrace(edt_ch0, TR_EVENTSLP);
      EdtTrace(edt_ch0, arg32);

      if (edt_ch0->m_EventMode == EDT_EVENT_MODE_SERIALIZE)
        edt_enable_remote_interrupt(edt_ch0);

#ifdef _WINDK_DRIVER_

      if (os.Events[arg32] && m_EventRefs[arg32])
      {
        os.Events[arg32]->Wait(TRUE, UserMode, UserRequest);
      }

#else
      retval = edt_wait_on_event(edt_ch0, arg32);
#endif
    }
    EdtTrace(edt_ch0, TR_EVENTWK);
    EdtTrace(edt_ch0, arg32);
    if (edt_ch0->m_EventCounters[arg32] > edt_ch0->m_EventWaitCount[arg32])
    {
      EdtTrace(edt_ch0, TR_EVENTWARN);
      EdtTrace(edt_ch0, edt_ch0->m_EventCounters[arg32]);
      EdtTrace(edt_ch0, edt_ch0->m_EventWaitCount[arg32]);
      EDTPRT(edt_ch0,
             DBG_IOCTLS,
             TRL_INFO,
             ("debug event %d count %d wait %d - reset\n",
              arg32,
              edt_ch0->m_EventCounters[arg32],
              edt_ch0->m_EventWaitCount[arg32]));
      /* Should we add this? - windows resyncs in user space */
      /* if (edt_ch0->m_EventSync)*/
      edt_ch0->m_EventWaitCount[arg32] = edt_ch0->m_EventCounters[arg32];
    }
    break;
  }

  case ES_STARTACT:
    edt_p->m_StartAction = arg32;
    if (edt_p->m_StartAction == EDT_ACT_ONCE || edt_p->m_StartAction == EDT_ACT_ALWAYS)
    {
      edt_p->m_DoStartDma = 1;
    }

    break;

  case ES_ENDACT:
    edt_p->m_EndAction = arg32;
    if (edt_p->m_EndAction == EDT_ACT_ONCE || edt_p->m_EndAction == EDT_ACT_ALWAYS)
    {
      edt_p->m_DoEndDma = 1;
    }
    break;

  case ES_RESETCOUNT:
    edt_p->m_timeout = 0;
    edt_p->m_Overflow = 0;
    break;

  case EG_TMSTAMP:
    if (edt_p->m_Numbufs)
    {
      int64_t* outbuf = (int64_t*)argBuf;
      int64_t bufnum = ((int64_t*)argBuf)[0];
      rbuf* p = &edt_p->m_Rbufs[bufnum % edt_p->m_Numbufs];

      outbuf[0] = edt_p->m_Done;
      outbuf[1] = p->m_end_timeval.tv_sec;
      outbuf[2] = p->m_end_timeval.tv_nsec;
    }
    else
      *bytesReturned = 0;
    break;

#ifdef PDV
  case ES_RESETSERIAL:
  {
    if (EDT_DEVID_IS_PDV(edt_p->devid))
    {
      edt_start_critical_src(edt_p, 37);
      if (pdvdrv_serial_init(edt_p) == 0)
      {
        EDTPRT(edt_p, DBG_SERIAL, TRL_INFO, ("Reset serial\n"));
        edt_enable_remote_interrupt(edt_p);
      }
      edt_end_critical_src(edt_p, 37);
    }
  }
#endif

  break;

  case ES_REFTMSTAMP:
    EdtTrace(edt_p, TR_REFTMSTAMP);
    EdtTrace(edt_p, arg32);

    EdtTraceTimeStamp(edt_p, NULL, 2);

    break;

  case EG_TODO:
    arg32 = edt_p->m_Todo;
    break;

  case ES_EVENT_HNDL:

#ifdef _NT_DRIVER_

    GET_EDT_BUF(edt_p, ep);

    EDTPRT(edt_p,
           DBG_IOCTLS,
           TRL_INFO,
           ("Set Event Handle %d -> %p\n", ep->desc, (void*)ep->value));

    edt_set_event_handle(edt_p, ep->desc, (void*)ep->value);

#ifdef _WINDK_DRIVER_
#if !defined(WINDK_ENV_WDM)
    {
      NTSTATUS status;
      GET_EDT_BUF(edt_p, ep);
      EDTPRT(edt_p, DBG_IOCTLS, TRL_VERBOSE, ("EVENT_HNDL %x for %x\n", ep->value, ep->desc));
      HANDLE hEvent = NULL;
      status =
          ObReferenceObjectByHandle((void*)ep->value, SYNCHRONIZE, NULL, KernelMode, &hEvent, NULL);
      EDTPRT(edt_p, DBG_IOCTLS, TRL_VERBOSE, ("buf event %x status %x\n", hEvent, status));

      if (edt_p->os.Events[ep->desc])
        delete edt_p->os.Events[ep->desc];
      edt_p->os.Events[ep->desc] = new CEvent((PKEVENT)hEvent);
      if (edt_p->os.Events[ep->desc])
        edt_p->os.Events[ep->desc]->Clear();
    }
#endif

#endif
    PUT_EDT_BUF(edt_p, ep);
#endif

    break;

#if defined(PCD)
  case ES_ABORTDMA_ONINTR:
  {
    /*
     * This is implemented to abort PCD DMA on STAT1 for TI.
     * It is not limited to this purpose.  -- Mark
     */
    edt_p->m_AbortDma_OnIntr = arg32;
    break;
  }
#endif

#ifdef PDV
  case ES_FVAL_DONE:
    pdvdrv_set_fval_done(edt_p, arg8);
    break;

  case EG_FVAL_DONE:
    arg8 = edt_p->m_DoneOnFval;
    break;

  case EG_LINES_XFERRED:
  {
    if (arg32 < edt_p->m_Numbufs)
      arg32 = edt_p->m_Rbufs[arg32].m_LinesXferred;
    else
      arg32 = 0;
  }
  break;
#endif /* PDV */

  case ES_PROCESS_ISR:
  {
#ifdef _NT_DRIVER_
    if (arg32 != (u_int)-1)
      edt_p->m_process_in_isr = (u_char)arg32;
    else
      arg32 = edt_p->m_process_in_isr;
#endif
  }
  break;

#if defined(PCD)
  case EG_MEZZ_ID:
    GET_EDT_BUF(edt_p, ep);

    userout(&dev0_p->mezz, (void*)ep->value, sizeof(EdtMezzDescriptor));

    PUT_EDT_BUF(edt_p, ep);
    break;

  case ES_MEZZ_ID:
    GET_EDT_BUF(edt_p, ep);
    userin((void*)ep->value, &dev0_p->mezz, sizeof(EdtMezzDescriptor));
    break;
#endif /* PCD */

  case ES_INDIRECT_REG_BASE:

    /* Base of indirect register access registers */
    if (arg32 == 0)
      edt_p->m_IndRegBase = 0x60;
    else
      edt_p->m_IndRegBase = arg32;
    break;

  case EG_INDIRECT_REG_BASE:
    arg32 = edt_p->m_IndRegBase;
    break;

  case ES_INTR_MASK:
    edt_set_interrupt_mask(edt_p, arg32);
    break;

  case EG_INTR_MASK:
    arg32 = edt_get_interrupt_mask(edt_p);
    break;

  default:
    retval = edt_device_ioctl(edt_p, controlCode, inSize, outSize, argBuf, bytesReturned, pIrp);
    return (retval);
  }

  if (retval)
  {
    *bytesReturned = 0;
  }
  else
  {
    if (argBuf == 0)
    {
      EDTPRT(edt_p, DBG_IOCTLS, TRL_INFO, ("code %x out argBuf 0\n", controlCode));
      retval = 1;
      *bytesReturned = 0;
    }
    else
      switch (outSize)
      {
      case 1:
        *(u_char*)argBuf = arg8;
        break;
      case 2:
        *(u_short*)argBuf = arg16;
        break;
      case 4:
        *(u_int*)argBuf = arg32;
        break;
      default:
        break;
      }
  }
  return retval;
}
