/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @file
 * @internal
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 */
#ifndef EDT_DRIVER_IOCTL_H
#define EDT_DRIVER_IOCTL_H

#include "edtdef.h"
#include "edt_ioctl.h"

#ifdef __cplusplus
extern "C" {
#endif

/* For kernel defines, use this shorthand. */
#define EMAPI(x) EDT_MAKE_IOCTL(EDT_DEVICE_TYPE, EIODA(x))

#define ES_DEBUG EMAPI(EDTS_DEBUG)
#define EG_DEBUG EMAPI(EDTG_DEBUG)
#define ES_REG   EMAPI(EDTS_REG)
#define EG_REG   EMAPI(EDTG_REG)

#define ES_PROG_READBACK EMAPI(EDTS_PROG_READBACK)
#define EG_DEBUG_SIZE    EMAPI(EDTG_DEBUG_SIZE)
#define EG_DEBUG_INFO    EMAPI(EG_DEBUG_INFO)
#define ES_SERIAL        EMAPI(EDTS_SERIAL)
#define EG_SERIAL        EMAPI(EDTG_SERIAL)
#define ES_DEPENDENT     EMAPI(EDTS_DEPENDENT)
#define EG_DEPENDENT     EMAPI(EDTG_DEPENDENT)
#define EG_DEVID         EMAPI(EDTG_DEVID)
#define ES_RTIMEOUT      EMAPI(EDTS_RTIMEOUT)
#define ES_WTIMEOUT      EMAPI(EDTS_WTIMEOUT)
#define EG_BUFDONE       EMAPI(EDTG_BUFDONE)
#define ES_NUMBUFS       EMAPI(EDTS_NUMBUFS)
#define ES_BUF           EMAPI(EDTS_BUF)
#define ES_BUF_MMAP      EMAPI(EDTS_BUF_MMAP)
#define ES_STARTBUF      EMAPI(EDTS_STARTBUF)
#if defined(__linux__)
#define ES_WAITBUF EMAPI(EDTS_WAITBUF)
#endif
#define ES_FREEBUF  EMAPI(EDTS_FREEBUF)
#define ES_STOPBUF  EMAPI(EDTS_STOPBUF)
#define ES_SETBUF   EMAPI(EDTS_SETBUF)
#define EG_TIMEOUTS EMAPI(EDTG_TIMEOUTS)
#define EG_TRACEBUF EMAPI(EDTG_TRACEBUF)
#define ES_STARTDMA EMAPI(EDTS_STARTDMA)
#define ES_ENDDMA   EMAPI(EDTS_ENDDMA)

// SERIAL_FIFO IOCTLs removed 6/2024, not fully working before, now handled in user mode.
// #define ES_SERIAL_FIFO EMAPI(EDTS_SERIAL_FIFO)
// #define EG_SERIAL_FIFO EMAPI(EDTG_SERIAL_FIFO)

#define EG_RTIMEOUT EMAPI(EDTG_RTIMEOUT)
#define EG_WTIMEOUT EMAPI(EDTG_WTIMEOUT)

#define ES_SERIALWAIT       EMAPI(EDTS_SERIALWAIT)
#define EG_OVERFLOW         EMAPI(EDTG_OVERFLOW)
#define ES_AUTODIR          EMAPI(EDTS_AUTODIR)
#define ES_FIRSTFLUSH       EMAPI(EDTS_FIRSTFLUSH)
#define EG_CONFIG_COPY      EMAPI(EDTG_CONFIG_COPY)
#define ES_CONFIG           EMAPI(EDTS_CONFIG)
#define EG_CONFIG           EMAPI(EDTG_CONFIG)
#define EG_SGTODO           EMAPI(EDTG_SGTODO)
#define EG_SGLIST           EMAPI(EDTG_SGLIST)
#define ES_SGLIST           EMAPI(EDTS_SGLIST)
#define EG_SGINFO           EMAPI(EDTG_SGINFO)
#define EG_TIMECOUNT        EMAPI(EDTG_TIMECOUNT)
#define ES_STARTACT         EMAPI(EDTS_STARTACT)
#define ES_ENDACT           EMAPI(EDTS_ENDACT)
#define ES_RESETCOUNT       EMAPI(EDTS_RESETCOUNT)
#define ES_RESETSERIAL      EMAPI(EDTS_RESETSERIAL)
#define ES_CLR_EVENT        EMAPI(EDTS_CLR_EVENT)
#define ES_ADD_EVENT_FUNC   EMAPI(EDTS_ADD_EVENT_FUNC)
#define ES_DEL_EVENT_FUNC   EMAPI(EDTS_DEL_EVENT_FUNC)
#define ES_WAIT_EVENT       EMAPI(EDTS_WAIT_EVENT)
#define EG_TMSTAMP          EMAPI(EDTG_TMSTAMP)
#define ES_TIMEOUT_ACTION   EMAPI(EDTS_TIMEOUT_ACTION)
#define EG_TIMEOUT_GOODBITS EMAPI(EDTG_TIMEOUT_GOODBITS)
#define ES_REG_OR           EMAPI(EDTS_REG_OR)
#define ES_REG_AND          EMAPI(EDTS_REG_AND)
#define EG_GOODBITS         EMAPI(EDTG_GOODBITS)
#define ES_BURST_EN         EMAPI(EDTS_BURST_EN)
#define EG_BURST_EN         EMAPI(EDTG_BURST_EN)
#define ES_DOTIMEOUT        EMAPI(EDTS_DOTIMEOUT)
#define ES_REFTMSTAMP       EMAPI(EDTS_REFTMSTAMP)
#define EG_BUFBYTECOUNT     EMAPI(EDTG_BUFBYTECOUNT)
#if defined(PDV) && (PDV != 0)
#define ES_PDVCONT  EMAPI(EDTS_PDVCONT)
#define ES_PDVDPATH EMAPI(EDTS_PDVDPATH)
#endif
#define ES_RESET_EVENT_COUNTER EMAPI(EDTS_RESET_EVENT_COUNTER)
#define EG_TODO                EMAPI(EDTG_TODO)
#define ES_RESUME              EMAPI(EDTS_RESUME)
#define ES_EVENT_HNDL          EMAPI(EDTS_EVENT_HNDL)
#define ES_MAX_BUFFERS         EMAPI(EDTS_MAX_BUFFERS)
#define EG_MAX_BUFFERS         EMAPI(EDTG_MAX_BUFFERS)
#define ES_PROG_XILINX         EMAPI(EDTS_PROG_XILINX)

#ifdef _WIN32
#define ES_MAPMEM   EMAPI(EDTS_MAPMEM)
#define ES_UNMAPMEM EMAPI(EDTS_UNMAPMEM)
#endif

#define ES_DRV_BUFFER EMAPI(EDTS_DRV_BUFFER)
#define ES_BITPATH    EMAPI(EDTS_BITPATH)
#define EG_BITPATH    EMAPI(EDTG_BITPATH)
#define EG_VERSION    EMAPI(EDTG_VERSION)
#define EG_BUILDID    EMAPI(EDTG_BUILDID)
#if defined(PCD)
#define ES_DIRECTION EMAPI(EDTS_DIRECTION)
#endif
#define EG_MEMSIZE  EMAPI(EDTG_MEMSIZE)
#define EG_MEM2SIZE EMAPI(EDTG_MEM2SIZE)

#if defined(PCD)
#define ES_ABORTDMA_ONINTR EMAPI(EDTS_ABORTDMA_ONINTR)
#endif

#ifdef PDV
#define ES_FVAL_DONE EMAPI(EDTS_FVAL_DONE)
#define EG_FVAL_DONE EMAPI(EDTG_FVAL_DONE)

// WAITCHAR IOCTL removed 6/2024, now handled in user mode.
// #define ES_WAITCHAR      EMAPI(EDTS_WAITCHAR)

#define EG_LINES_XFERRED EMAPI(EDTG_LINES_XFERRED)
#endif

#define ES_PROCESS_ISR EMAPI(EDTS_PROCESS_ISR)
#define ES_CLEAR_DMAID EMAPI(EDTS_CLEAR_DMAID)
#ifdef PDV
#define EG_SERIAL_WRITE_AVAIL EMAPI(EDTG_SERIAL_WRITE_AVAIL)
#endif

#define EG_WAIT_STATUS      EMAPI(EDTG_WAIT_STATUS)
#define ES_WAIT_STATUS      EMAPI(EDTS_WAIT_STATUS)
#define ES_TIMEOUT_OK       EMAPI(EDTS_TIMEOUT_OK)
#define EG_TIMEOUT_OK       EMAPI(EDTG_TIMEOUT_OK)
#define ES_MULTI_DONE       EMAPI(EDTS_MULTI_DONE)
#define EG_MULTI_DONE       EMAPI(EDTG_MULTI_DONE)
#define ES_TEST_LOCK_ON     EMAPI(EDTS_TEST_LOCK_ON)
#define ES_MEZZ_BITPATH     EMAPI(EDTS_MEZZ_BITPATH)
#define EG_MEZZ_BITPATH     EMAPI(EDTG_MEZZ_BITPATH)
#define EG_DMA_INFO         EMAPI(EDTG_DMA_INFO)
#define ES_TRACE_REG        EMAPI(EDTS_TRACE_REG)
#define ES_REG_BIT_CLEARSET EMAPI(EDTS_REG_BIT_CLEARSET)
#define ES_REG_BIT_SETCLEAR EMAPI(EDTS_REG_BIT_SETCLEAR)
#if defined(PCD)
#define ES_MEZZ_ID EMAPI(EDTS_MEZZ_ID)
#define EG_MEZZ_ID EMAPI(EDTG_MEZZ_ID)
#endif /* PCD */
#define EG_NUMBUFS EMAPI(EDTG_NUMBUFS)

#define ES_INDIRECT_REG_BASE EMAPI(EDTS_INDIRECT_REG_BASE)
#define EG_INDIRECT_REG_BASE EMAPI(EDTG_INDIRECT_REG_BASE)

#define ES_DEBUG_MASK    EMAPI(EDTS_DEBUG_MASK)
#define EG_DEBUG_MASK    EMAPI(EDTG_DEBUG_MASK)
#define ES_ALLOC_KBUFFER EMAPI(EDTS_ALLOC_KBUFFER)

#define ES_INTR_MASK EMAPI(EDTS_INTR_MASK)
#define EG_INTR_MASK EMAPI(EDTG_INTR_MASK)

#ifdef __cplusplus
}
#endif

#endif /* EDT_DRIVER_IOCTL_H */
