/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (c) 2022 Engineering Design Corporation, Inc.
 */

#ifndef EDT_DRIVER_VERSION_H
#define EDT_DRIVER_VERSION_H

#if defined(_WIN32)
#include <basetsd.h>
typedef UINT64 uint64_t;
typedef UINT32 uint32_t;
#elif defined(__linux__)
#include <linux/types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct edt_driver_version_s {
  uint32_t major;           /**< Major version number */
  uint32_t minor;           /**< Minor version number */
  uint32_t patch;           /**< Patch version number */
  const char* prerelease;   /**< Pre-release string, such as "rc0" */
  uint32_t vcs_id;          /**< Internal version control system id */
  uint64_t build_timestamp; /**< Build time, as UNIX timestamp */
} edt_driver_version_t;

const edt_driver_version_t* edt_driver_get_version(void);

/** EDT driver version string */
extern const char EDT_DRIVER_VERSION[];

/**
 * Number of bytes in @ref EDT_DRIVER_VERSION.
 * @note uses sizeof() not strlen()
 */
extern const uint32_t EDT_DRIVER_VERSION_LEN;

/** EDT driver build id string */
extern const char EDT_DRIVER_BUILDID[];

/**
 * Number of bytes in @ref EDT_DRIVER_BUILDID.
 * @note uses sizeof() not strlen()
 */
extern const uint32_t EDT_DRIVER_BUILDID_LEN;

#ifdef __cplusplus
} /* extern C */
#endif

#endif /* EDT_DRIVER_VERSION_H */
