/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 */

#ifndef EDT_MEM_MAPPING_H
#define EDT_MEM_MAPPING_H

#ifdef __linux__
#include <linux/types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned long kernel_addr_t;

typedef unsigned int bus_addr32_t; /* current PCI bus always 32 bit addresses */

typedef void* user_addr_t;

/* structure needed for DMA sg list */

typedef struct {
  bus_addr32_t busaddr;
  u_int count;
} EdtPCISg;

/* linked list for creating pools of pages */

typedef struct _EdtMemList {
  struct _EdtMemList* prev;
  struct _EdtMemList* next;
  void* page;
  void* vaddr;          /* kernel virtual address */
  bus_addr32_t busaddr; /* physical address */
  u_short memsize;      /* size as allocated i.e. 4096 */
  u_short size_used;    /* size in use */
  u_char mapped;        /* is mapped  - busaddr is valid */
} EdtMemList;

/* memory map info for a ring buffer */

typedef struct {
  EdtPCISg* addr;        /* virtual address of sg page */
  bus_addr32_t physaddr; /* bus address of sg page */
  unsigned int size;
  unsigned int entries; /* size / sizeof(EdtPCISg) */
} EdtMemRange;

typedef struct {
  unsigned long size;

  void** page_addresses; /* array of kernel virtual addresses for pages */

  void* vma; /* hold vma for get_user_pages */

  unsigned short npages; /* number of pages in user space */
  unsigned short offset;
  unsigned short sg_entries_needed;
  unsigned short sg_pages_allocated;
  unsigned char user_mem;
  unsigned char own_pages;
  unsigned char is_mapped;
  unsigned char rw;
  unsigned char is_dma_mapped;

  void* os_sglist;  /* pointer to os-specific sglist */
  int sglist_count; /* value returned by OS routine
                       (may collapse sg list in Linux) */

  int pgsize;
  int sgsize;

  struct pci_dev* pci_dev; /* pointer to device */

  EdtMemList** page_list_ptrs; /* list of allocated sg pages */

  EdtMemList** sg_list_ptrs; /* list of dma-able sg pages */

  EdtPCISg* chunks; /* active list of sg page bus address/counts */
                    /* The busaddr in this list is used in SetupRdy */

  unsigned int nalloc_chunks; /*number of sg pages allocated */
  unsigned int nchunks;       /* number of sg pages actually used */

} EdtMemMapping;

#ifdef __cplusplus
} /* extern C */
#endif

#endif /* EDT_MEM_MAPPING_H */
