/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2023 Engineering Design Team, Inc.
 */

#ifndef _EDT_MEM_TAG_H_
#define _EDT_MEM_TAG_H_

#define MEMTAG_SIZE 0x40000

typedef struct {
  void* addr;
  unsigned int size;
  int tagid;
} EdtMemTag;

typedef struct {
  unsigned int total_allocated;
  unsigned int max_allocated;
  char name[64];

  EdtMemTag tags[MEMTAG_SIZE];
  unsigned int entries;
  unsigned int max_entries;
  unsigned int first_empty;
  unsigned int last_empty;
  unsigned int total;

} EdtMemTagPool;

extern EdtMemTagPool* kmalloc_tags;
extern EdtMemTagPool* vmalloc_tags;
extern EdtMemTagPool* page_tags;
extern EdtMemTagPool* page_cache_tags;

// #define DEBUG_LEAKS

#ifdef DEBUG_LEAKS

asmlinkage void add_mem_tag(EdtMemTagPool* pool, const void* addr, const int size, const char* tag);

asmlinkage void clear_mem_tag(EdtMemTagPool* pool, const void* addr, const char* tag);

asmlinkage void init_mem_pools(void);
asmlinkage void dump_mem_pools(void);
asmlinkage void clear_mem_pools(void);

#else

// define away functions if not tagging

#define add_mem_tag(pool, addr, size, tag) (void)tag

#define clear_mem_tag(pool, addr, tag) (void)tag
#endif

#endif
