/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2023 Engineering Design Team, Inc.
 * @brief IDs of EDT "PCI Digital Vision" (PDV) class devices.
 */

#ifndef EDT_PCI_DEVICES_PDV_H
#define EDT_PCI_DEVICES_PDV_H

#ifdef __cplusplus
extern "C" {
#endif

/* X-macro of EDT PCI devices and their properties
 *
 * X(_defname, _pid, _dma_ch, _name, _desc)
 *
 * - PCI PID define name
 * - PCI PID value
 * - Number of DMA channels
 * - Name (Short, lowercase, underscores, no spaces. Useful as filenames.)
 * - Description
 */
#define EDT_PCI_DEVICES_LISTX(X)                           \
  X(PE1DVVL_ID, 0x009dU, 3, "pe1g2_vlf1", "VisionLink F1") \
  X(PE4DVVL_ID, 0x00a3U, 3, "pe4g2_vlf4", "VisionLink F4") \
  X(PE8VLCLS_ID, 0x00a9U, 2, "pe8g2_vlcls", "VisionLink CLS")

/*-----------------------------------------------------------------------------
 * "ID_IS" macros
 *---------------------------------------------------------------------------*/

#define EDT_DEVID_IS_PDV(id) ((id == PE1DVVL_ID) || (id == PE4DVVL_ID) || (id == PE8VLCLS_ID))

#define EDT_DEVID_IS_1LANE(id) ((id == PE1DVVL_ID))

#define EDT_DEVID_IS_4LANE(id) ((id == PE4DVVL_ID))

#define EDT_DEVID_IS_8LANE(id) ((id == PE8VLCLS_ID))

#define EDT_DEVID_IS_DVCL(id) ((id == PE1DVVL_ID) || (id == PE4DVVL_ID))

/* PCIe Camera Link simulators only  */
#define EDT_DEVID_IS_DVCLS(id) ((id == PE8VLCLS_ID))

/* All Camera Link simulators */
#define EDT_DEVID_IS_CLSIM(id) ((EDT_DEVID_IS_DVCLS(id)))

#define EDT_DEVID_HAS_IRIGB(id) ((id == PE1DVVL_ID) || (id == PE4DVVL_ID))

/* ADD any devices that don't have a separate interface FPGA here */
#define EDT_DEVID_HAS_COMBINED_FPGA(id) \
  ((id == PE8VLCLS_ID) || (id == PE1DVVL_ID) || (id == PE4DVVL_ID))

#define EDT_DEVID_HAS_16BIT_PROM(id) (0 == 1) /* false */

#define EDT_DEVID_IS_MICRON_PROM(id) \
  ((id == PE1DVVL_ID) || (id == PE4DVVL_ID) || (id == PE8VLCLS_ID))

#define EDT_DEVID_PCILOAD_INFO_NA(id) ((id == PE8VLCLS_ID))

#ifdef __cplusplus
} /* extern C */
#endif

#endif /* EDT_PCI_DEVICES_PDV_H */
