/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @internal
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 */

#include "edtdrv.h"

u_int GoodBits(Edt_Dev* edt_p)
{
  u_int stat;
  u_int bytenum;
  u_int bitnum;

  stat = edt_get(edt_p, PCD_STAT);
  bytenum = (stat & (SSDIO_BYTECNT_MSK)) >> SSDIO_BYTECNT_SHFT;
  bitnum = (stat & 0x38) >> 3;
  return (bytenum * 8 + bitnum);
}

void edt_timeout(void* arg_p)
{
  u_int count = 0;
  u_int cfg;
  u_int fifocnt;
  Edt_Dev* edt_p = (Edt_Dev*)arg_p;

  if (!edt_p->m_TimeoutOK)
  {
    edt_p->m_WaitStatus = EDT_WAIT_TIMEOUT;
  }
  else
  {
    edt_p->m_WaitStatus = EDT_WAIT_OK_TIMEOUT;
    return;
  }

  EDTPRT(edt_p,
         DBG_TIMEOUT,
         TRL_INFO,
         ("timeout Edt_Dev %p channel %d unit %d", edt_p, edt_p->m_DmaChannel, edt_p->m_Unitno));

  edt_start_critical_src(edt_p, 51);

  edt_p->m_timeout++;

  if (edt_p->m_TimeoutAction == EDT_TIMEOUT_BIT_STROBE)
  {
    u_int stat;
    u_int tmp;
    u_int bytenum;
    u_int bitnum;

    stat = edt_get(edt_p, PCD_STAT);

    edt_p->m_TimeoutGoodBits = GoodBits(edt_p);

    if (edt_p->m_TimeoutGoodBits)
    {
      stat = edt_get(edt_p, PCD_STAT);
      EDTPRT(edt_p, DBG_TIMEOUT, TRL_INFO, ("wait till get up to last bit"));
      while ((stat & 7) != 7)
      {
        bytenum = (stat & (SSDIO_BYTECNT_MSK)) >> SSDIO_BYTECNT_SHFT;
        bitnum = (stat & 0x38) >> 3;
        EDTPRT(edt_p,
               DBG_TIMEOUT,
               TRL_INFO,
               ("%x bit %d byte %d fifo %x", stat, bitnum, bytenum, edt_fifocount(edt_p)));
        tmp = edt_get(edt_p, PCD_FUNCT);

        edt_set(edt_p, PCD_FUNCT, tmp & ~SSDIO_STROBE);
        /* this just to add delay and flush cache */
        stat = edt_get(edt_p, PCD_STAT);
        edt_set(edt_p, PCD_FUNCT, tmp | SSDIO_STROBE);

        stat = edt_get(edt_p, PCD_STAT);
      }
      EDTPRT(edt_p, DBG_TIMEOUT, TRL_INFO, ("stat1 %x", stat));
      EDTPRT(edt_p, DBG_TIMEOUT, TRL_INFO, ("now should be one more"));
      while ((stat & 7) == 7)
      {
        bytenum = (stat & (SSDIO_BYTECNT_MSK)) >> SSDIO_BYTECNT_SHFT;
        bitnum = (stat & 0x38) >> 3;
        EDTPRT(edt_p,
               DBG_TIMEOUT,
               TRL_INFO,
               ("%x bit %d byte %d fifo %x", stat, bitnum, bytenum, edt_fifocount(edt_p)));
        tmp = edt_get(edt_p, PCD_FUNCT);

        edt_set(edt_p, PCD_FUNCT, tmp & ~SSDIO_STROBE);
        /* this just to add delay and flush cache */
        stat = edt_get(edt_p, PCD_STAT);
        edt_set(edt_p, PCD_FUNCT, tmp | SSDIO_STROBE);

        stat = edt_get(edt_p, PCD_STAT);
      }
      EDTPRT(edt_p, DBG_TIMEOUT, TRL_INFO, ("stat2 %x", stat));
    }

    stat = edt_get(edt_p, PCD_STAT);
    EDTPRT(edt_p,
           DBG_TIMEOUT,
           TRL_INFO,
           ("Done.. Stat =%x Good bits = %d pci fifo %d\n",
            stat,
            edt_p->m_TimeoutGoodBits,
            edt_fifocount(edt_p)));
  }

  count = TransferCount(edt_p);
  edt_p->m_timecount = count;

  EDTPRT(edt_p, DBG_TIMEOUT, TRL_INFO, ("m_timecount %d", edt_p->m_timecount));

  cfg = edt_get(edt_p, EDT_DMA_CFG);
  if (cfg & EDT_FIFO_FLG)
  {
    fifocnt = ((cfg & EDT_FIFO_CNT) >> EDT_FIFO_SHIFT) + 1;
    fifocnt *= 4;
  }
  else
    fifocnt = 0;

  if (fifocnt)
  {
    cfg = edt_get(edt_p, EDT_DMA_CFG);
    edt_set(edt_p, EDT_DMA_INTCFG_ONLY, ((cfg | EDT_WRITE_STROBE) >> 8) & 0xff);
    EDTPRT(edt_p, DBG_TIMEOUT, TRL_INFO, ("fifocnt %d", fifocnt));
  }

  edt_p->m_Done++;

  edt_p->m_had_timeout = 1;
  edt_abortdma(edt_p);
  EdtTrace(edt_p, TR_TIMEOUT);

  EdtTrace(edt_p, count);

#if defined(__linux__)
  edt_wakeup_dma(edt_p);
#endif

  edt_end_critical_src(edt_p, 0x201);
}

/*
 * Abbreviated edt_timeout callable from interrupt level.
 * Must be called from within a critical section.   Mark/Amit 05/07
 */
void edt_timeout_intr(void* arg_p)
{
  u_int count;
  u_int cfg;
  u_int fifocnt;
  Edt_Dev* edt_p = (Edt_Dev*)arg_p;

  edt_p->m_WaitStatus = EDT_WAIT_TIMEOUT;
  edt_p->m_timeout++;

  count = TransferCount(edt_p);
  edt_p->m_timecount = count;

  cfg = edt_get(edt_p, EDT_DMA_CFG);
  if (cfg & EDT_FIFO_FLG)
  {
    fifocnt = ((cfg & EDT_FIFO_CNT) >> EDT_FIFO_SHIFT) + 1;
    fifocnt *= 4;
  }
  else
    fifocnt = 0;

  if (fifocnt)
  {
    cfg = edt_get(edt_p, EDT_DMA_CFG);
    edt_set(edt_p, EDT_DMA_INTCFG_ONLY, ((cfg | EDT_WRITE_STROBE) >> 8) & 0xff);
    EDTPRT(edt_p, DBG_TIMEOUT, TRL_INFO, ("fifocnt %d", fifocnt));
  }

  edt_p->m_Done++;
  edt_p->m_had_timeout = 1;
  edt_abortdma(edt_p);

#if defined(__linux__)
  edt_wakeup_dma(edt_p);
#endif
}
