/* SPDX-License-Identifier: BSD-3-Clause OR GPL-2.0-only */

/*!
 * @internal
 * @file
 * @author EDT <tech@edt.com>
 * @copyright Copyright (C) 1997 Engineering Design Team, Inc.
 */

#include "edtdrv.h"

#include "pcd.h"

#include "edt_driver_ioctl.h"

/*
 * hooks to map driver dependencies into driver independent code.
 */

int pcd_device_ioctl(Edt_Dev* edt_p,
                     u_int controlCode,
                     u_int inSize,
                     u_int outSize,
                     void* argBuf,
                     u_int* bytesReturned,
                     void* UNUSED_pIrp)
{
  u_int retval = 0;
  u_char arg8 = 0;
  u_short arg16 = 0;
  u_int arg32 = 0;

  (void)UNUSED_pIrp;

  if (argBuf == 0)
    EDTPRINTF(edt_p, 0, ("code %x in argBuf 0\n", controlCode));
  else
    switch (inSize)
    {
    case 1:
      arg8 = *(u_char*)argBuf;
      break;
    case 2:
      arg16 = *(u_short*)argBuf;
      break;
    case 4:
      arg32 = *(u_int*)argBuf;
      break;
    default:
      break;
    }

  *bytesReturned = outSize;

  /* check both incase ioctl called instead of edt_ioctl */
  switch (controlCode)
  {
  case ES_ADD_EVENT_FUNC:
  {
    edt_start_critical_src(edt_p, 40);

    switch (arg32)
    {
    case EDT_EVENT_PCD_STAT1:
      edt_or(edt_p, PCD_CMD, PCD_STAT_INT_1 | PCD_ENABLE);
      break;
    case EDT_EVENT_PCD_STAT2:
      edt_or(edt_p, PCD_CMD, PCD_STAT_INT_2 | PCD_ENABLE);
      break;
    case EDT_EVENT_PCD_STAT3:
      edt_or(edt_p, PCD_CMD, PCD_STAT_INT_3 | PCD_ENABLE);
      break;
    case EDT_EVENT_PCD_STAT4:
      edt_or(edt_p, PCD_CMD, PCD_STAT_INT_4 | PCD_ENABLE);
      break;
    case EDT_EVENT_TEMP:

      edt_or(edt_p, EDT_DMA_INTCFG_ONLY, (EDT_TEMP_EN_INTR >> 8) & 0xff);
      EDTPRINTF(edt_p, 0, ("Setting Temp Intr cfg = %x\n", edt_get(edt_p, EDT_DMA_INTCFG)));
      break;

    default:
      EDTPRINTF(edt_p, 0, ("ES_ADD_EVENT_FUNC default"));
      break;
    }
    edt_or(edt_p, PCD_STAT_POLARITY, PCD_STAT_INT_ENA);
    edt_enable_remote_interrupt(edt_p);

    edt_end_critical_src(edt_p, 40);

    break;
  }

  case ES_DEL_EVENT_FUNC:
  {
    edt_start_critical_src(edt_p, 41);

    switch (arg32)
    {
    case EDT_EVENT_PCD_STAT1:
      edt_clear(edt_p, PCD_CMD, PCD_STAT_INT_1);
      break;
    case EDT_EVENT_PCD_STAT2:
      edt_clear(edt_p, PCD_CMD, PCD_STAT_INT_2);
      break;
    case EDT_EVENT_PCD_STAT3:
      edt_clear(edt_p, PCD_CMD, PCD_STAT_INT_3);
      break;
    case EDT_EVENT_PCD_STAT4:
      edt_clear(edt_p, PCD_CMD, PCD_STAT_INT_4);
      break;
    case EDT_EVENT_TEMP:
      edt_clear(edt_p, EDT_DMA_INTCFG_ONLY, (EDT_TEMP_EN_INTR >> 8) & 0xff);
      break;

    default:
      EDTPRINTF(edt_p,
                0,
                ("ES_DEL_EVENT_FUNC default: controlCode 0x%x arg32 0x%x", controlCode, arg32));
      break;
    }

    edt_end_critical_src(edt_p, 41);

    break;
  }

  case ES_CLR_EVENT:
    break;

  default:
    EDTPRINTF(edt_p, 0, ("pcd : ioctl %x", controlCode));
    retval = 1;
    break;
  }

  if (retval)
    *bytesReturned = 0;
  else
  {
    if (argBuf == 0)
      EDTPRINTF(edt_p, 0, ("code %x out argBuf 0\n", controlCode));
    else
      switch (outSize)
      {
      case 1:
        *(u_char*)argBuf = arg8;
        break;
      case 2:
        *(u_short*)argBuf = arg16;
        break;
      case 4:
        *(u_int*)argBuf = arg32;
        break;
      default:
        break;
      }
  }

  return (retval);
}

/*
 * edt_hwctl():  interface to device dependent hardware operations.
 */

/* TODO - move to unit struct */

int pcd_hwctl(int operation, void* context, Edt_Dev* edt_p)
{
  switch (operation)
  {
  case INIT_DEVICE:
    edt_InitializeBoard(edt_p);
    break;
  case LAST_CLOSE:
  {
    /* if (edt_p->m_DmaActive)*/
    {
      u_int sg_nxt_cnt = edt_get(edt_p, EDT_SG_NXT_CNT);
      u_int cmd;

      EDTPRINTF(edt_p, 1, ("last close abort"));
      cmd = sg_nxt_cnt & (EDT_DMA_MEM_RD | EDT_BURST_EN);
      cmd |= (EDT_DMA_ABORT /* | EDT_EN_SG_DONE*/);
      edt_p->m_Done = edt_p->m_Todo = edt_p->m_Loaded;
      edt_p->m_Freerun = 0;
      edt_p->m_CurSg = 0;

      edt_set_sg_cnt(edt_p, cmd);

      edt_p->m_DmaActive = 0;

      /*
      cv_broadcast(&edt_p->m_signalcv);
      */
    }

    /* no break ; */
  }
  case IDLE_DEVICE:
  {
    break;
  }

  case ABORT_DMA:

    break;

  case DISABLE_INTR:

    break;

  case TEST_INTR:
    break;

  case ACK_INTR:
    break;

  case GET_DMA_RESID:
    break;

  case LOAD_REGISTERS:
    break;

  case DMA_COUNTDOWN:
    break;

  case EDT_DMA_ADDRESS:
    break;

  case MMAP_REGISTERS:
    break;

  case POSTIO:
    break;

  case STARTIO:
    break;

  case PRE_STARTIO_OP:
    break;

  case FIRST_OPEN:
  {
    break;
  }

  default:
    EDTPRINTF(edt_p, 0, ("Unknown edt_hwctl() operation 0x%x context 0x%p", operation, context));
    break;
  }

  return (0);
}

int pcd_DeviceInt(Edt_Dev* edt_p)
{
  u_int status = edt_get(edt_p, PCD_STAT);
  u_int dstatus = edt_get(edt_p, PCD_DATA_PATH_STAT);
  int ret = 0;
  if (dstatus & PCD_OVERFLOW)
  {
    edt_p->m_Overflow++;
  }

  if (edt_p->m_savestat & EDT_TEMP_INTR)
  {
    EDTPRINTF(edt_p, 1, ("Temp interrupt\n"));

    edt_broadcast_event(edt_p, EDT_EVENT_TEMP);
  }

  if (status & PCD_STAT_INT_X)
  {
    EdtTrace(edt_p, 0xbbbb0000);

    // In serialized mode rmt interrupts are enabled upon next wait.
    if (edt_p->m_EventMode == EDT_EVENT_MODE_SERIALIZE)
    {
      edt_disable_remote_interrupt(edt_p);
    }

    if (status & PCD_STAT_INT_1)
    {
      EdtTrace(edt_p, 0xbbbb0001);
      edt_clear(edt_p, PCD_CMD, PCD_STAT_INT_1);

#if defined(PCD)
      if (edt_p->m_AbortDma_OnIntr)
      {
        u_int sg_nxt_cnt = edt_get(edt_p, EDT_SG_NXT_CNT);

        if ((sg_nxt_cnt & 0x40c00000) != 0x40c00000)
        {
          /*
           * Force device not ready (DNR) true for TI -- Mark 5/28/03
           * Turn off when DMA (read/write) returns.
           */
          edt_p->m_AbortDma_OnIntr_errno = ECONNRESET;

          if (edt_p->m_AbortDma_OnIntr == 2)
          {
            edt_or(edt_p, PCD_CONFIG, 0x20);
          }

          edt_timeout_intr(edt_p);
        }
      }
#endif /* PCD */

      if (edt_p->m_EventRefs[EDT_EVENT_PCD_STAT1])
      {
        EdtTrace(edt_p, 0xbbbb0011);
        edt_or(edt_p, PCD_CMD, PCD_STAT_INT_1);
        edt_p->m_event_flags[EDT_EVENT_PCD_STAT1] = 1;
        edt_broadcast_event(edt_p, EDT_EVENT_PCD_STAT1);
      }
    }
    if (status & PCD_STAT_INT_2)
    {
      EdtTrace(edt_p, 0xbbbb0002);
      edt_clear(edt_p, PCD_CMD, PCD_STAT_INT_2);

      if (edt_p->m_EventRefs[EDT_EVENT_PCD_STAT2])
      {
        EdtTrace(edt_p, 0xbbbb0022);
        edt_or(edt_p, PCD_CMD, PCD_STAT_INT_2);
        edt_p->m_event_flags[EDT_EVENT_PCD_STAT2] = 1;
        edt_broadcast_event(edt_p, EDT_EVENT_PCD_STAT2);
      }
    }
    if (status & PCD_STAT_INT_3)
    {
      EdtTrace(edt_p, 0xbbbb0003);
      edt_clear(edt_p, PCD_CMD, PCD_STAT_INT_3);

      EdtTrace(edt_p, edt_p->m_Unitno);
      if (edt_p->m_EventRefs[EDT_EVENT_PCD_STAT3])
      {
        EdtTrace(edt_p, 0xbbbb0033);
        edt_or(edt_p, PCD_CMD, PCD_STAT_INT_3);
        edt_p->m_event_flags[EDT_EVENT_PCD_STAT3] = 1;
        edt_broadcast_event(edt_p, EDT_EVENT_PCD_STAT3);
      }
    }
    if (status & PCD_STAT_INT_4)
    {
      EdtTrace(edt_p, 0xbbbb0004);
      edt_clear(edt_p, PCD_CMD, PCD_STAT_INT_4);

      if (edt_p->m_EventRefs[EDT_EVENT_PCD_STAT4])
      {
        EdtTrace(edt_p, 0xbbbb0044);
        edt_or(edt_p, PCD_CMD, PCD_STAT_INT_4);
        edt_p->m_event_flags[EDT_EVENT_PCD_STAT4] = 1;
        edt_broadcast_event(edt_p, EDT_EVENT_PCD_STAT4);
      }
    }

    edt_enable_remote_interrupt(edt_p);

    ret = 1;
  }

  return (ret);
}
void pcd_dep_init(Edt_Dev* UNUSED_edt_p)
{
  (void)UNUSED_edt_p;
}
