/* SPDX-License-Identifier: BSD-3-Clause */

/**
 * @file
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 */
#ifndef EDT_OS_LINUX_H
#define EDT_OS_LINUX_H

#include "edt_portability.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <limits.h>
#include <stdint.h>
#include <errno.h>

#include <dirent.h>
#include <semaphore.h>
#include <sys/user.h>
#include <pthread.h>

#ifdef __cplusplus
extern "C" {
#endif

EDT_DEPRECATED_TYPE("6.2.0",
                    "Replace with standard C99/C11 types, such as 'unsigned [int]' or 'uint32_t'.")
typedef unsigned int uint_t;

EDT_DEPRECATED_TYPE("6.2.0",
                    "Replace with standard C99/C11 types, such as 'unsigned char' or 'uint8_t'.")
typedef unsigned char uchar_t;

EDT_DEPRECATED_TYPE("6.2.0",
                    "Replace with standard C99/C11 types, such as 'unsigned short' or 'uint16_t'")
typedef unsigned short ushort_t;

/* <sys/types.h> will define these old BSD types if _GNU_SOURCE or some old
 * BSD or POSIX "feature test" macro was defined. Earlier EDT v5.x packages used
 * these types significantly. Now this EDT SDK is moving towards C11 syntax
 * without GNU extensions, preferring standard integer types.
 *
 * If the user or the compiler did not ask for these types to be defined,
 * define them for now but recommend users use something else.
 */
#if !(defined(_GNU_SOURCE) || defined(_BSD_SOURCE) || defined(__USE_MISC) || \
      defined(__u_char_defined))
EDT_DEPRECATED_TYPE("6.2.0",
                    "Replace BSD types with standard C99/C11 types, or define the appropriate "
                    "feature test macro for your system.")
typedef unsigned char u_char;

EDT_DEPRECATED_TYPE("6.2.0",
                    "Replace BSD types with standard C99/C11 types, or define the appropriate "
                    "feature test macro for your system.")
typedef unsigned short u_short;

EDT_DEPRECATED_TYPE("6.2.0",
                    "Replace BSD types with standard C99/C11 types, or define the appropriate "
                    "feature test macro for your system.")
typedef unsigned int u_int;

EDT_DEPRECATED_TYPE("6.2.0",
                    "Replace BSD types with standard C99/C11 types, or define the appropriate "
                    "feature test macro for your system.")
typedef unsigned long int u_long;
#endif

/* utility helpers */

#ifndef TRUE
#define TRUE  (1)
#define FALSE (0)
#endif

typedef int HANDLE;

#define DIRHANDLE   DIR*
#define NULL_HANDLE ((HANDLE)-1)

#ifndef PAGE_SIZE
#include <unistd.h>
#define PAGE_SIZE sysconf(_SC_PAGESIZE)
#endif

#define PAGESIZE    PAGE_SIZE
#define SECTOR_SIZE PAGE_SIZE

#ifdef _POSIX_PATH_MAX
#define MAX_PATH _POSIX_PATH_MAX
#else
#define MAX_PATH 256
#endif

#ifdef PATH_MAX
#define MAXPATH PATH_MAX
#else
#define MAXPATH 4096
#endif

typedef pthread_mutex_t edt_critical_section_t;

#ifdef __cplusplus
}
#endif

#endif /* EDT_OS_LINUX_H */
