/* SPDX-License-Identifier: BSD-3-Clause */

/**
 * @file
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 */

#ifndef EDT_TRACE_H
#define EDT_TRACE_H

#include "edt_portability.h"

#ifndef _KERNEL
#include <stdio.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define EDT_TRACESIZE 1024 /* in longs */

/* Driver Debugging */

#define TRL_NONE 0

#define TRL_CRITICAL    1
#define TRL_FATAL       1
#define TRL_ERROR       2
#define TRL_WARN        3
#define TRL_INFO        4
#define TRL_VERBOSE     5
#define TRL_REALVERBOSE 6
#define TRL_RESERVED7   7
#define TRL_RESERVED8   8
#define TRL_RESERVED9   9

#define EDT_DEBUG_LEVELS 10

/* Driver trace mask bits */

#define DBG_INIT         0x00000100
#define DBG_PNP          0x00000200
#define DBG_POWER        0x00000400
#define DBG_CREATE_CLOSE 0x00001000
#define DBG_IOCTLS       0x00002000
#define DBG_WRITE        0x00004000
#define DBG_READ         0x00008000
#define DBG_DPC          0x00010000
#define DBG_ISR          0x00020000
#define DBG_LOCKS        0x00040000
#define DBG_EVENTS       0x00080000
#define DBG_HW           0x00100000
#define DBG_REG          0x00200000
#define DBG_DMA          0x00400000
#define DBG_DMA_SETUP    0x00800000
#define DBG_SERIAL       0x01000000
#define DBG_BOARD        0x02000000
#define DBG_ALLOC        0x04000000
#define DBG_MEMMAP       0x08000000
#define DBG_TIME         0x10000000
#define DBG_TIMEOUT      0x20000000
#define DBG_INTR_EN      0x40000000
#define DBG_ALL          0xffffff00

#define N_DBG_STATES 24

/* The debug level is in the lower 8 bits of a debug mask */

#define DBG_MASK_VAL(x)       ((x)&0xffffff00)
#define DBG_MASK_LEVEL(x)     ((x)&0xff)
#define DBG_MASK(level, mask) ((mask) | (level & 0xff))

#define DBG_READWRITE (DBG_READ | DBG_WRITE)

/* kernel driver trace support */

#ifndef _KERNEL
enum edt_trace_filter_mode { EDT_TRACE_FILTER_INCLUDE = 0, EDT_TRACE_FILTER_EXCLUDE = 1 };

EDTAPI void edt_trace(EdtDev edt_p,
                      FILE* output,
                      int doraw,
                      int loops,
                      unsigned int* filter,
                      enum edt_trace_filter_mode filter_mode);

EDTAPI void edt_trace_file(FILE* input,
                           FILE* output,
                           unsigned int* filter,
                           enum edt_trace_filter_mode filter_mode);
EDTAPI void edt_dump_registers(EdtDev edt_p, int debug);
#endif /* _KERNEL */

#ifdef __cplusplus
}
#endif

#endif /* EDT_TRACE_H */
