# Example Applications

SPDX-License-Identifier: BSD-3-Clause

Copyright (C) 2022 Engineering Design Team, Inc.

## Building Examples

### Linux

This example demonstrates using CMake v3.10.2 and GCC 7.5 on an Ubuntu 18.04 system.

The commands below create a new build directory and generate a CMake configuration using Unix Makefiles in a "Release" configuration.

```bash
mkdir build && cd build

cmake -G "Unix Makefiles" -DCMAKE_BUILD_TYPE=Release ../
```

Then, build all examples using the configuration in this build directory:

```bash
cmake --build .
```

The resulting binaries can be found in this build directory.

### Windows with Visual Studio

This example demonstrates using CMake v3.20.2 and Visual Studio 16 on Windows 10.

This below command generates a CMake configuration using Visual Studio 16 in a directory named `build`.

```cmd
cmake -B build -G "Visual Studio 16 2019"
```

Then, using the "Release" configuration, build all examples:

```cmd
cmake --build build --config Release
```

The resulting binaries can be found in the build directory: `examples/build/Release/`.

### Windows with MinGW-w64

This example demonstrates using CMake v3.20.2 and MinGW-w64 on Windows 10.

- Visit the downloads page of the MinGW-w64 project:
  https://www.mingw-w64.org/downloads/

- Find "MinGW-W64-builds" option in the table. This just says that releases are
  available to download directly from GitHub.
  https://github.com/niXman/mingw-builds-binaries/releases

- Choose the "win32-seh-ucrt" variant.

- Picking GCC 13.2.0, so will download `x86_64-13.2.0-release-win32-seh-ucrt-rt_v11-rev1.7z` from
  https://github.com/niXman/mingw-builds-binaries/releases/tag/13.2.0-rt_v11-rev1

- Open and extract with 7Zip to a location without spaces in the path,
  for example `C:/mingw64_13.2.0/`.

- Open a PowerShell terminal and append the `/bin` directory of MinGW to the `PATH` variable.
  Don't forget the semicolon path separator.

  ```powershell
  > $env:PATH += ";C:\mingw64_13.2.0\bin"
  ```
- Configure CMake to generate build scripts (in the current directory) using MinGW

  ```powershell
  > cmake -G "MinGW Makefiles" -S C:\EDT\pdv\examples -B ./ -DCMAKE_BUILD_TYPE=Release
  ```
- Finally, build all example targets

  ```powershell
  > cmake --build ./
  ```
