/* SPDX-License-Identifier: BSD-3-Clause */

/**
 * @file
 * @copyright Copyright (C) 2011 Engineering Design Team, Inc.
 * PCI DV Dependent structure and defines
 */
#ifndef PDV_DEPENDENT_H
#define PDV_DEPENDENT_H

#include "libpdv.h"
#include "edt_portability.h"

#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Opaque pointer. */
typedef struct pdv_dependent* PdvDependent;

/* General purpose reorder descriptor
 * Tap starts at startx, starty. Sample at dx,dy for length pixels.
 * Go to startx, starty + stridex,stridey.
 */
typedef struct pdv_tap_descriptor {
  int startx; /* PDV_DIM_END means at end of row */
  int starty; /* PDV_DIM_END means at last row */
  int dx;
  int dy;
  int length;
  int stridex; /* PDV_DIM_END means at end of row */
  int stridey; /* PDV_DIM_END means at last row */
} PdvInterleaveTap;

#define CAM_NAME_LEN  128
#define FNAME_LEN     256
#define MAX_INIT      1024
#define OLD_MAX_INIT  256
#define KBS_FNAME_LEN 64
#define CAM_CLASS_LEN 64

#define MAX_SERIAL          32
#define MAX_XILINX_REG_LIST 32

#define MAX_INTERLEAVE_TAPS 8

#define PDV_DIM_END -1
#define PDV_MAXTAPS 8

#define KODAK_AIA_MCL       1 /* mode control (typ. CC1) pulse-width */
#define AIA_MCL             1 /* same as KODAK */
#define IRC_160             2 /* IRC 160 */
#define AIA_SERIAL          3 /* trigger pulse (1 millisecond) */
#define KODAK_AIA_SER       3 /* dup */
#define AIA_SER             3 /* dup */
#define KODAK_XHF_INTLC     4 /* horizontal interlace */
#define PDV_BYTE_INTLV      4 /* horizontal interlace */
#define KODAK_XHF_SKIP      5 /* subsample */
#define PDV_BYTE_INTLV_SKIP 5 /* subsample */
#define IMAGE_MERGE         6 /* merge */
#define PDV_FIELD_INTLC     6 /* merge */
#define HW_ONLY             7
#define ES10_WORD_INTLC     8  /* need to deinterlace words */
#define PDV_WORD_INTLV      8  /* need to deinterlace words */
#define PDV_BGGR            9  /* expermental */
#define PDV_BGGR_DUAL       10 /* expermental */
#define PDV_BGGR_WORD       11 /* expermental */
#define PDV_BYTE_TEST1      12 /* temporary */
#define PDV_BYTE_TEST2      13 /* temporary */
#define PDV_BYTE_TEST3      14 /* temporary */
#define KODAK_SER_14I       21 /* Kodak chgd their AIA syntax */
#define HAM_4880_SER        22 /* AIA serial controlled camera */
#define PULNIX_TM1000       23 /* AIA_MCL controlled camera */
#define PULNIX_TM9700       24 /* AIA_MCL controlled camera */
#define KODAK_DIG_GAIN      25 /* AIA_MCL w/digital gain */
// #define FOI_REMOTE_AIA         26 /* AIA Remote Camera Interface */
#define KODAK_AIA_SER_CTRL     27 /* AIA serial but use shutter ctrl */
#define AIA_SER_CTRL           27
#define DALSA_CONTINUOUS       28 /* DALSA D71024T continuous mode */
#define HAM_4880_8X            29 /* AIA serial controlled camera */
#define HAM_4742_95            30 /* AIA serial controlled camera */
#define PROGRES                31
#define AIA_MC4                32 /* MCL gain only for photonics gain */
#define SMD_SERIAL             33 /* SMD binary serial method */
#define AIA_SERIAL_ES40        34 /* Kodak modified AIA */
#define PDV_DALSA_4CH_INTLV    35 /* Dalsa four channel interleave (frame) */
#define SPECINST_SERIAL        36 /* Spectral Instruments */
#define PDV_INVERT_RIGHT_INTLV 37 /* even pixels are right half, inverted */
#define PDV_DALSA_2CH_INTLV    PDV_INVERT_RIGHT_INTLV
#define PDV_WORD_INTLV_ODD     38 /* sameas WORD_INTLV but odd ln 1st */
#define PDV_DDCAM              39 /* EDT/Delta Design camera */
#define PDV_DALSA_LS           40
#define PDV_HEADER_BEFORE      41 /* header data comes before img data */
#define PDV_HEADER_AFTER       42 /* header data comes after img data */
#define PDV_HEADER_WITHIN      43 /* header data is within def'd img */
/* size - use hdr offs to dtmn where */
#define PDV_WORD_INTLV_HILO      44 /* deintrlace words, high/low */
#define PDV_SPECINST_4PORT_INTLV 45 /* Spectral Instruments 4 port intlv */
#define PDV_QUADRANT_INTLV       46 /* quadrant intlv */
#define PDV_ILLUNIS_INTLV        47 /* illunis quadrant intlv */
#define PDV_ILLUNIS_BGGR         48 /* illunis quadrant intlv + bggr */

#define PDV_ES10_BGGR               49 /* Roper Horizontal Intlc w/ bayer */
#define PDV_ES10_WORD_BGGR          50 /* Roper Horizontal Intlc w/ bayer */
#define PDV_ES10_WORD_ODD_BGGR      51 /* Roper Horizontal Intlc w/ bayer */
#define AIA_MCL_100US               52 /* AIA_MCL w/100 microsec decade ctr */
#define PDV_PIRANHA_4CH_INTLV       53 /* Piranha four channel interleave */
#define PDV_FMRATE_ENABLE           54 /* Frame Rate Enable flag */
#define PDV_FVAL_ADJUST             55 /* Frame Valid Adjust flag */
#define BASLER_202K                 56 /* Basler A202K */
#define DUNCAN_2131                 57 /* Basler A202K */
#define PDV_EVEN_RIGHT_INTLV        58 /* even pixels are right half */
#define PDV_DALSA_2M30_INTLV        PDV_EVEN_RIGHT_INTLV
#define PDV_INV_RT_INTLV_24_12      59 /* clink 24-12 DALSA_2CH_INLTV */
#define PDV_INTLV_24_12             60 /* clink 24-12  */
#define PDV_INTLV_1_8_MSB7          61 /* one-bit , pixel 0 at lsb */
#define PDV_INTLV_1_8_MSB0          62 /* one-bit , pixel 0 at msb */
#define PDV_INTLV_BGR_2_RGB         63 /* BGR -> RGB */
#define PDV_INTLV_USER              64 /* look up  */
#define SU320_SERIAL                65 /* Sensor Unlmited 320M serial expose method */
#define BASLER202K_SERIAL           66 /* Basler 202K serial expose method */
#define ADIMEC_SERIAL               67 /* Adimec serial expose method */
#define TIMC1001_SERIAL             68 /* TI MC1001 serial expose method */
#define PTM6710_SERIAL              69 /* Pulnix TM-6710 serial expose method */
#define PTM1020_SERIAL              70 /* Pulnix TM-1020 serial expose method */
#define AIA_TRIG                    71 /* new: explicit triggered mode, sets mode_cntl automatically */
#define PDV_PIRANHA_4CH_HWINTLV     72 /* piranha 4 channel deinterleave IN HARDWARE */
#define PDV_INVERT_RIGHT_BGGR_INTLV 73 /* even pixels are right half, inverted BAYER pattern*/

#define PDV_LINE_INTLV        74
#define PDV_GENERIC_INTLV     75
#define PDV_LINE_INTLV_P3_8X4 76
#define TOSHIBA_SERIAL        77
#define COHU_SERIAL           78

#define PDV_WORD_INTLV_TOPBOTTOM 80
#define PDV_BYTE_INTLV_TOPBOTTOM 81
#define PDV_WORD_INTLV_INOUT     82
#define PDV_BYTE_INTLV_INOUT     83

#define PDV_DALSA_LS_4CH_INTLV       84 /* Dalsa four channel interleave (linescan) */
#define PDV_WORD_INTLV_HILO_LINE     85 /*PCO egde mode A*/
#define PDV_WORD_INTLV_TOPMID_LINE   86 /*PCO edge mode D */
#define PDV_BYTE_INTLV_MIDTOP_LINE   87 /*hamamatsu OrcaFlash4.0 */
#define PDV_WORD_INTLV_MIDTOP_LINE   88 /*hamanatsu OrcaFlash4.0 */
#define PDV_INTLV_10BIT_8TAP_PACKED  89
#define PDV_INTLV_10BIT_8TAP_TO_8BIT 90
#define PDV_INTLV_20BAND             91 /* custom 21 ... */
#define PDV_INTLV_21BAND             92 /* and 21-band   */

#define PDV_QUADRANT2_INTLV 93 /* 4 port deintlv, TopLeft, TopMiddle, MiddleLeft, MIddleMiddle */
#define PDV_QUADRANT3_INTLV \
  94 /* 4 port deintlv, like QUADRANT_INTLV but starting in the middle, iterating out */

#define KODAK_RDM_SINGLE 1
#define KODAK_RDM_DUAL   2

#define PDV_FRAMESYNC_OFF             0
#define PDV_FRAMESYNC_ON              1
#define PDV_FRAMESYNC_EMULATE_TIMEOUT 2

/* serial_format */
#define SERIAL_ASCII          0
#define SERIAL_BINARY         1
#define SERIAL_PULNIX_1010    2
#define SERIAL_ASCII_HEX      3
#define SERIAL_BASLER_FRAMING 4
#define SERIAL_DUNCAN_FRAMING 5
#define SERIAL_ASCII_NOSPACE  6
#define SERIAL_TOSHIBA        7

/* SMD registers common to all models (we hope!) */
#define SMD_READ_CAMTYPE 0xc3
#define SMD_READ_FWREV   0xc5
#define SMD_RESET        0x80

/* camera type, returned from SMD_READ_CAMTYPE register */
#define SMD_TYPE_4M4   0x14
#define SMD_TYPE_BT25  0x38
#define SMD_TYPE_1M30P 0x41
#define SMD_TYPE_1M15P 0x11
#define SMD_TYPE_6M3P  0x33

/* from SMD 4M4 Camera Interface Specification */
#define SMD_4M4_WRITE_R1       0x81
#define SMD_4M4_WRITE_R2       0x82
#define SMD_4M4_WRITE_R3       0x84
#define SMD_4M4_WRITE_R4       0x88
#define SMD_4M4_READ_R1        0xc1
#define SMD_4M4_READ_R2        0xc2
#define SMD_4M4_READ_R3        0xc4
#define SMD_4M4_READ_R4        0xc8
#define SMD_4M4_READ_FRAMERATE 0xca

/* 4M4 (and 1M30P) reg bits (only partial, see manual for complete list) */
#define SMD_4M4_R1_INTEGRATE 0x00 /* integrate mode, 0=ext, 1=int */
#define SMD_4M4_R1_TRIG      0x02 /* trigger mode, 0=ext, 1=cont */
#define SMD_4M4_R1_BINMSK    0x0c /* bin mode, 00=1x1, 01=2x2, 10=4x4 */
#define SMD_4M4_R1_GAIN      0x10 /* gain -- 0 = 1x, 1 = 4x */
#define SMD_4M4_BIN_2X2      0x04
#define SMD_4M4_BIN_4X4      0x08

/* SMD 1M30P is pretty much the same as 4M4 */
#define SMD_1M30P_REG_W_ADC_RESET 0x00
#define SMD_1M30P_REG_W_LS_OFFSET 0x02
#define SMD_1M30P_REG_W_MS_OFFSET 0x03
#define SMD_1M30P_REG_W_LS_GAIN   0x05
#define SMD_1M30P_REG_W_MS_GAIN   0x06

#define SMD_1M30P_REG_R_LS_OFFSET 0x42
#define SMD_1M30P_REG_R_MS_OFFSET 0x43
#define SMD_1M30P_REG_R_LS_GAIN   0x45
#define SMD_1M30P_REG_R_MS_GAIN   0x46

#define SMD_1M30P_REG_W_CAM_RESET 0x80
#define SMD_1M30P_REG_W_CONTROL   0x82
#define SMD_1M30P_REG_W_BINNING   0x85
#define SMD_1M30P_REG_W_INTEG0    0x8A
#define SMD_1M30P_REG_W_INTEG1    0x8B
#define SMD_1M30P_REG_W_INTEG2    0x8C
#define SMD_1M30P_REG_W_FRAME0    0x8D
#define SMD_1M30P_REG_W_FRAME1    0x8E
#define SMD_1M30P_REG_W_FRAME2    0x8F

#define SMD_1M30P_REG_R_CONTROL 0xc2

/* from SMD BT25 Camera Interface Specification */
#define SMD_BT25_WRITE_R1       0x81
#define SMD_BT25_WRITE_R2       0x82
#define SMD_BT25_WRITE_R3       0x84
#define SMD_BT25_WRITE_R4       0x88
#define SMD_BT25_READ_R1        0xc1
#define SMD_BT25_READ_R2        0xc2
#define SMD_BT25_READ_R3        0xc4
#define SMD_BT25_READ_R4        0xc8
#define SMD_BT25_READ_FRAMERATE 0xca

/* from DALSTAR 1M15P Camera Interface Specification */
#define SMD_1M15P_WRITE_R1 0x82
#define SMD_1M15P_WRITE_R2 0x84
#define SMD_1M15P_WRITE_R3 0x88
#define SMD_1M15P_READ_R1  0xc2
#define SMD_1M15P_READ_R2  0xc4
#define SMD_1M15P_READ_R3  0xc8

/* 1M15P regigster bits */
#define SMD_1M15P_R1_INTMODE   0x80 /* integrate mode, 0=ext, 1=int */
#define SMD_1M15P_R1_GAIN      0x40 /* gain, 0 = 1x, 1 = 4x */
#define SMD_1M15P_R1_ANTIBLOOM 0x20 /* anti-bloom, 0=enabled 1=disabled */
#define SMD_1M15P_R1_BINMODE   0x10 /* bin mode, 00=1x1, 01=2x2, 10=4x4 */
#define SMD_1M15P_R1_TRIGMODE  0x08 /* trigger mode, 0=ext, 1=cont */
#define SMD_1M15P_R1_INTMSK    0x03 /* integration time bits */

/* compat */
#define HAM_4880           HAM_4880_SER
#define HAMAMATSU_4880     HAM_4880_SER
#define HAMAMATSU_4880_SER HAM_4880_SER
#define HAMAMATSU_4880_8X  HAM_4880_8X

#define PDV_NOLOCKDEV 0
#define PDV_LOCKDEV   1

#define PDV_SERIAL_RS422 0
#define PDV_SERIAL_RS232 1

/* Front panel value */
#define FRONT_PANEL     0x7fffffff
#define FACTORY_SETTING 0x7ffffff0

/* LH Strobe Methods */
#define PDV_LHS_METHOD1 1
#define PDV_LHS_METHOD2 2

#define NOT_SET -9999999 /* doubtful it would ever be a valid value */

/**
 * @ingroup init
 *
 * @brief Allocates a PdvDependent structure, for use by @ref pdv_readcfg and
 * @ref pdv_initcam, and checks for and reports error conditions as a result of
 * the alloc.
 *
 * The structure should be deallocated with free() later.
 *
 * @return Pointer to a PdvDependent structure, or NULL if there was an error.
 * @see @ref pdv_initcam, @ref pdv_readcfg
 */
EDT_DEALLOCATE_WITH(free)
EDTAPI PdvDependent pdv_alloc_dependent(void);

/**
 * @brief Clear all structure members to zero.
 *
 * @param p
 *    Pointer to a PdvDependent structure.
 */
EDTAPI void pdv_dependent_erase(PdvDependent p);

/**
 * @brief Read PdvDependent data from driver.
 *
 * @param pdv_p
 *   The open PDV device handle.
 * @param[out] dep
 *   On success, data read from driver will be returned here.
 *
 * @return 0 on success.
 */
EDTAPI int pdv_get_dependent(PdvDev pdv_p, PdvDependent dep);

/**
 * @brief Write PdvDependent data to driver.
 *
 * @param pdv_p
 *   The open PDV device handle.
 * @param dep
 *   Data to store in the device driver handle.
 *
 * @return 0 on success.
 */
EDTAPI int pdv_set_dependent(PdvDev pdv_p, PdvDependent dep);

#ifdef __cplusplus
}
#endif

#endif /* PDV_DEPENDENT_H */
